// author: Kwang-Hyun Park (akaii@kw.ac.kr)

#include <stdio.h>
#include <conio.h>
#include "roboid.h"

#define NUM_HAMSTERS 4

// Example00: Scan
void example_scan(void) {
	scan(); // print serial ports
}

// Example01-1: Basic Movement
void example_basic_movement1(void) {
	hamster_s_create();

	// move forward/backward 3.5cm, turn left/right 50.5 degrees with 60% velocity
	hamster_s_move_forward(3.5, 60);
	hamster_s_move_backward(3.5, 60);
	hamster_s_turn_left(50.5, 60);
	hamster_s_turn_right(50.5, 60);

	// move forward/backward, turn left/right for 1.5 seconds with 60% velocity
	hamster_s_move_forward_sec(1.5, 60);
	hamster_s_move_backward_sec(1.5, 60);
	hamster_s_turn_left_sec(1.5, 60);
	hamster_s_turn_right_sec(1.5, 60);

	// move forward/backward, turn left/right for 1.5 seconds with 60% velocity
	hamster_s_wheels(60, 60);
	wait(1500);
	hamster_s_wheels(-60, -60);
	wait(1500);
	hamster_s_wheels(-60, 60);
	wait(1500);
	hamster_s_wheels(60, -60);
	wait(1500);

	// stop
	hamster_s_stop();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example01-2: Basic Movement
void example_basic_movement2(void) {
	hamster_s_create();

	// pivot around left wheel 90/-90 degrees with 60% velocity
	hamster_s_pivot_left_wheel(90, 60);
	hamster_s_pivot_left_wheel(-90, 60);

	// pivot around right wheel 90/-90 degrees with 60% velocity
	hamster_s_pivot_right_wheel(90, 60);
	hamster_s_pivot_right_wheel(-90, 60);

	// pivot around left wheel 90 degrees with 60/-60% velocity
	hamster_s_pivot_left_wheel(90, 60);
	hamster_s_pivot_left_wheel(90, -60);

	// pivot around right wheel 90 degrees with 60/-60% velocity
	hamster_s_pivot_right_wheel(90, 60);
	hamster_s_pivot_right_wheel(90, -60);

	// pivot around left wheel for 1.5 seconds with 60/-60% velocity
	hamster_s_pivot_left_wheel_sec(1.5, 60);
	hamster_s_pivot_left_wheel_sec(1.5, -60);

	// pivot around right wheel for 1.5 seconds with 60/-60% velocity
	hamster_s_pivot_right_wheel_sec(1.5, 60);
	hamster_s_pivot_right_wheel_sec(1.5, -60);

	// pivot around left wheel for 1.5 seconds with 60/-60% velocity
	hamster_s_wheels(0, 60);
	wait(1500);
	hamster_s_wheels(0, -60);
	wait(1500);

	// pivot around right wheel for 1.5 seconds with 60/-60% velocity
	hamster_s_wheels(60, 0);
	wait(1500);
	hamster_s_wheels(-60, 0);
	wait(1500);

	// stop
	hamster_s_stop();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example01-3: Basic Movement
void example_basic_movement3(void) {
	hamster_s_create();

	// pivot around left pen 90/-90 degrees with 60% velocity
	hamster_s_pivot_left_pen(90, 60);
	hamster_s_pivot_left_pen(-90, 60);

	// pivot around right pen 90/-90 degrees with 60% velocity
	hamster_s_pivot_right_pen(90, 60);
	hamster_s_pivot_right_pen(-90, 60);

	// pivot around left pen 90 degrees with 60/-60% velocity
	hamster_s_pivot_left_pen(90, 60);
	hamster_s_pivot_left_pen(90, -60);

	// pivot around right pen 90 degrees with 60/-60% velocity
	hamster_s_pivot_right_pen(90, 60);
	hamster_s_pivot_right_pen(90, -60);

	// pivot around left pen for 1.5 seconds with 60/-60% velocity
	hamster_s_pivot_left_pen_sec(1.5, 60);
	hamster_s_pivot_left_pen_sec(1.5, -60);

	// pivot around right pen for 1.5 seconds with 60/-60% velocity
	hamster_s_pivot_right_pen_sec(1.5, 60);
	hamster_s_pivot_right_pen_sec(1.5, -60);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example01-4: Basic Movement
void example_basic_movement4(void) {
	hamster_s_create();

	// left pen draws 90/-90 degrees of circle on the left with radius 5cm with 60% velocity
	hamster_s_left_pen_circle_left(90, 5, 60);
	hamster_s_left_pen_circle_left(-90, 5, 60);

	// left pen draws 90/-90 degrees of circle on the right with radius 5cm with 60% velocity
	hamster_s_left_pen_circle_right(90, 5, 60);
	hamster_s_left_pen_circle_right(-90, 5, 60);

	// right pen draws 90/-90 degrees of circle on the left with radius 5cm with 60% velocity
	hamster_s_right_pen_circle_left(90, 5, 60);
	hamster_s_right_pen_circle_left(-90, 5, 60);

	// right pen draws 90/-90 degrees of circle on the right with radius 5cm with 60% velocity
	hamster_s_right_pen_circle_right(90, 5, 60);
	hamster_s_right_pen_circle_right(-90, 5, 60);

	// body turns left 90/-90 degrees with radius 5cm with 60% velocity
	hamster_s_circle_left(90, 5, 60);
	hamster_s_circle_left(-90, 5, 60);

	// body turns right 90/-90 degrees with radius 5cm with 60% velocity
	hamster_s_circle_right(90, 5, 60);
	hamster_s_circle_right(-90, 5, 60);

	// left pen draws circle on the left/right for 2 seconds with radius 5cm with 60% velocity
	hamster_s_left_pen_circle_left_sec(2, 5, 60);
	hamster_s_left_pen_circle_right_sec(2, 5, 60);

	// left pen draws circle on the left/right for 2 seconds with radius 5cm with 60% velocity
	hamster_s_right_pen_circle_left_sec(2, 5, 60);
	hamster_s_right_pen_circle_right_sec(2, 5, 60);

	// body turns left/right for 2 seconds with radius 5cm with 60% velocity
	hamster_s_circle_left_sec(2, 5, 60);
	hamster_s_circle_right_sec(2, 5, 60);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example02: Simple Movement
void example_simple_movement(void) {
	int i;

	hamster_s_create();

	for(i = 0; i < 10; ++i) {
		hamster_s_move_forward(5, 60); // move forward 5cm with 60% velocity
		hamster_s_move_backward(5, 60); // move backward 5cm with 60% velocity
		hamster_s_turn_left(90, 60); // turn left 90 degrees with 60% velocity
	}
	hamster_s_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example03: Acceleration
void example_acceleration(void) {
	int i;

	hamster_s_create();
	
	// acceleration
	for(i = 0; i < 100; i += 2) {
		hamster_s_wheels(i, i);
		wait(20); // 20 msec
	}

	// 0.5 seconds
	wait(500);

	// deceleration
	for(i = 100; i >= 0; i -= 2) {
		hamster_s_wheels(i, i);
		wait(20); // 20 msec
	}

	hamster_s_stop();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example04: Led
void example_led(void) {
	hamster_s_create();

	hamster_s_rgbs(255, 0, 0, 0, 255, 0); // left: red and right: green
	wait(500); // 0.5 seconds

	hamster_s_left_rgb(255, 63, 0); // left: orange
	wait(500); // 0.5 seconds

	hamster_s_right_rgb(0, 255, 255); // right: sky blue
	wait(500); // 0.5 seconds

	hamster_s_leds("red", "green"); // turn on both LEDs
	wait(500); // 0.5 seconds

	hamster_s_left_led("off"); // turn off left LED
	wait(500); // 0.5 seconds

	hamster_s_left_led("blue"); // turn on left LED
	wait(500); // 0.5 seconds

	hamster_s_right_led("off"); // turn off right LED
	wait(500); // 0.5 seconds

	hamster_s_right_led("yellow"); // turn on right LED
	wait(500); // 0.5 seconds

	hamster_s_leds("off", "off"); // turn off both LEDs

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example05: Siren
void example_siren(void) {
	int i, hz;

	hamster_s_create();

	for(i = 0; i < 10; ++i) {
		for(hz = 500; hz <= 700; hz += 10) {
			hamster_s_buzzer(hz);
			wait(20); // 20 msec
		}
	}
	hamster_s_buzzer(0); // clear buzzer

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06: Melody
void example_melody(void) {
	int i, j;

	hamster_s_create();

	for(i = 0; i < 2; ++i) {
		for(j = 0; j < 2; ++j) {
			hamster_s_note("C4", 0.5);
			hamster_s_note("E4", 0.5);
			hamster_s_note("G4", 0.5);
		}

		for(j = 0; j < 3; ++j) {
			hamster_s_note("A4", 0.5);
		}

		hamster_s_note("G4", 1);
		hamster_s_note("off", 0.5);

		for(j = 0; j < 3; ++j) {
			hamster_s_note("F4", 0.5);
		}

		for(j = 0; j < 3; ++j) {
			hamster_s_note("E4", 0.5);
		}

		for(j = 0; j < 3; ++j) {
			hamster_s_note("D4", 0.5);
		}

		hamster_s_note("C4", 1);
		hamster_s_note("off", 0.5);

		hamster_s_tempo(120); // speed up
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example07: Sound
void example_sound1(void) {
	hamster_s_create();

	// play sound 2 times
	hamster_s_repeat_sound("beep", 2);
	wait(500);

	hamster_s_sound("random beep");
	wait(500);

	hamster_s_sound("siren");
	wait(500);

	hamster_s_sound("engine");
	wait(500);

	hamster_s_sound("robot");
	wait(500);

	hamster_s_sound("off");

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example08: Sound
void example_sound2(void) {
	hamster_s_create();

	// play sound and wait until done
	hamster_s_sound_until_done("siren");

	// random beep: 3 times
	hamster_s_repeat_sound_until_done("random beep", 3);

	hamster_s_sound_until_done("dibidibidip");

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example09: Sensor
void example_sensor(void) {
	int i;

	hamster_s_create();

	for(i = 0; i < 500; ++i) { // 10 seconds
		printf("Signal Strength: %d\n", hamster_s_signal_strength());
		printf("Proximity: %d, %d\n", hamster_s_left_proximity(), hamster_s_right_proximity());
		printf("Floor: %d, %d\n", hamster_s_left_floor(), hamster_s_right_floor());
		printf("Acceleration: %d, %d, %d\n", hamster_s_acceleration_x(), hamster_s_acceleration_y(), hamster_s_acceleration_z());
		printf("Light: %d\n", hamster_s_light());
		printf("Temperature: %d\n", hamster_s_temperature());
		printf("Input: %d, %d\n", hamster_s_input_a(), hamster_s_input_b());
		printf("Tilt: %d\n", hamster_s_tilt());
		printf("Battery State: %d\n\n", hamster_s_battery_state());

		wait(20); // 20 msec
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example10: Staggering Walk
void example_staggering_walk(void) {
	int acc, count;

	hamster_s_create();

	for(count = 10; count > 0; ) {
		hamster_s_wheels(-100, -100); // move backward
		acc = hamster_s_acceleration_x();
		if(acc > 2000 || acc < -2000) {
			hamster_s_wheels(100, 100); // move forward
			wait(250); // 0.25 seconds

			hamster_s_wheels(-50, 50); // turn left
			wait(500); // 0.5 seconds

			-- count;
		}
		wait(10); // 10 msec
	}
	hamster_s_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example11: Theremin Sound
void example_theremin_sound(void) {
	double hz = 0;
	int proximity, i;

	hamster_s_create();

	for(i = 0; i < 500; ++i) { // 10 seconds
		proximity = hamster_s_left_proximity();
		if(proximity < 10) {
			proximity = 0;
		}
		hz = (hz * 5 + proximity * 50) / 10.0;
		hamster_s_buzzer(hz);

		wait(20); // 20 msec
	}
	hamster_s_buzzer(0); // clear sound

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example12: Simple Hand Follower
void example_simple_hand_follower(void) {
	int proximity, left_speed, right_speed, i;

	hamster_s_create();

	for(i = 0; i < 500; ++i) { // 10 seconds
		// left wheel
		proximity = hamster_s_left_proximity();
		if(proximity > 15) {
			left_speed = (40 - proximity) * 4;
		} else {
			left_speed = 0;
		}

		// right wheel
		proximity = hamster_s_right_proximity();
		if(proximity > 15) {
			right_speed = (40 - proximity) * 4;
		} else {
			right_speed = 0;
		}

		hamster_s_wheels(left_speed, right_speed);
		wait(20); // 20 msec
	}
	hamster_s_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example13: Builtin Line Tracer
void example_builtin_line_tracer(void) {
	hamster_s_create();

	hamster_s_line_left(); // follow black line with left floor sensor
	wait(5000); // 5 seconds

	hamster_s_line_speed(8); // speed up
	wait(5000); // 5 seconds

	hamster_s_line_right(); // follow black line with right floor sensor
	wait(5000); // 5 seconds

	hamster_s_line_speed(5); // speed down
	wait(5000); // 5 seconds

	hamster_s_line_both(); // follow black line with both floor sensors
	wait(5000); // 5 seconds

	hamster_s_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example14: Builtin Line Tracer Intersection
void example_builtin_line_tracer_intersection(void) {
	hamster_s_create();
	
	// move to a front intersection
	hamster_s_cross_forward();

	// move to a left intersection
	hamster_s_cross_left();

	// move to a right intersection
	hamster_s_cross_right();

	// u-turn and move to an intersection
	hamster_s_cross_uturn();
	
	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example15: Hand Detector
void example_hand_detector(void) {
	hamster_s_create();

	// wait while the value of the left proximity is less than 30
	while(hamster_s_left_proximity() < 30) {
		wait(10); // 10 msec
	}

	// move backward 5cm with beep sound
	hamster_s_sound("beep");
	hamster_s_move_backward(5, 60);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example16: Move On Board
// This example requires a Hamster board (http://hamster.school/ko/tutorial/class or http://hamster.school/en/tutorial/class)
void example_move_on_board(void) {
	hamster_s_create();

	// move one space forward
	hamster_s_board_forward();

	// turn left once
	hamster_s_board_left();

	// move one space forward
	hamster_s_board_forward();

	// turn right once
	hamster_s_board_right();

	// move one space forward
	hamster_s_board_forward();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example17: Extension - Button
// This example requires a Hamster's extension kit
void example_extension_button(void) {
	int i;

	hamster_s_create();

	// set a port B to the digital input mode
	hamster_s_io_mode_b("digital input");

	for(i = 0; i < 500; ++i) { // 10 seconds
		if(hamster_s_input_b() == 0) {
			hamster_s_buzzer(1000);
		} else {
			hamster_s_buzzer(0);
		}
		wait(20); // 20 msec
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example18: Extension - Led Blink
// This example requires a Hamster's extension kit
void example_extension_led_blink(void) {
	int i;

	hamster_s_create();

	// set a port A to the digital output mode
	hamster_s_io_mode_a("digital output");

	for(i = 0; i < 5; ++i) {
		hamster_s_output_a(1); // turn on
		wait(1000); // 1 second

		hamster_s_output_a(0); // turn off
		wait(1000); // 1 second
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example19: Extension - Potentiometer
// This example requires a Hamster's extension kit
void example_extension_potentiometer(void) {
	int potentiometer, i;

	hamster_s_create();

	// set a port A to the analog input mode
	hamster_s_io_mode_a("analog input");

	for(i = 0; i < 500; ++i) { // 10 seconds
		potentiometer = hamster_s_input_a();
		hamster_s_buzzer(potentiometer);
		wait(20); // 20 msec
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example20: Extension - Smooth Led
// This example requires a Hamster's extension kit
void example_extension_smooth_led(void) {
	int i, j;

	hamster_s_create();

	// set a port A to the PWM output mode
	hamster_s_io_mode_a("pwm output");

	for(i = 0; i < 3; ++i) {
		// turn on
		for(j = 0; j <= 255; j += 5) {
			hamster_s_output_a(j);
			wait(20); // 20 msec
		}

		// turn off
		for(j = 255; j >= 0; j -= 5) {
			hamster_s_output_a(j);
			wait(20); // 20 msec
		}
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example21: Extension - Servo
// This example requires a Hamster's extension kit
void example_extension_servo(void) {
	int i;

	hamster_s_create();

	// set a port A to the servo output mode
	hamster_s_io_mode_a("servo output");

	for(i = 0; i < 3; ++i) {
		hamster_s_output_a(180); // move to 180 degrees
		wait(1000); // 1 second

		hamster_s_output_a(10); // move to home
		wait(1000); // 1 second
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example22: Extension - Voltage
// This example requires a Hamster's extension kit
void example_extension_voltage(void) {
	int i;

	hamster_s_create();

	// set a port A to the servo output mode
	hamster_s_io_mode_a("voltage input");

	for(i = 0; i < 200; ++i) { // 4 seconds
		printf("%d, %f\n", hamster_s_input_a(), hamster_s_voltage_a());
		wait(20); // 20 msec
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example23: Dual Theremin Sound
void example_dual_theremin_sound(void) {
	double hz1 = 0, hz2 = 0;
	int proximity, i;

	HamsterS* hamster1 = hamster_s_create();
	HamsterS* hamster2 = hamster_s_create();

	// wait until two robots are ready
	wait_until_ready();

	for(i = 0; i < 500; ++i) { // 10 seconds
		proximity = hamster1->left_proximity();
		if(proximity < 10) proximity = 0;
		hz1 = (hz1 * 5 + proximity * 50) / 10.0;
		hamster1->buzzer(hz1);

		proximity = hamster2->left_proximity();
		if(proximity < 10) proximity = 0;
		hz2 = (hz2 * 5 + proximity * 50) / 10.0;
		hamster2->buzzer(hz2);

		wait(20); // 20 msec
	}

	// clear sounds
	hamster1->buzzer(0);
	hamster2->buzzer(0);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example24: Mutiple Simple Movement
void example_multiple_basic_movement(void) {
	HamsterS* hamsters[NUM_HAMSTERS];
	int i, j;

	for(i = 0; i < NUM_HAMSTERS; ++i) {
		hamsters[i] = hamster_s_create();
	}

	// wait until four robots are ready
	wait_until_ready();

	for(i = 0; i < 10; ++i) {
		// move forward
		for(j = 0; j < NUM_HAMSTERS; ++j) {
			hamsters[j]->wheels(50, 50);
		}
		wait(500); // 0.5 seconds

		// move backward
		for(j = 0; j < NUM_HAMSTERS; ++j) {
			hamsters[j]->wheels(-50, -50);
		}
		wait(500); // 0.5 seconds

		// turn left
		for(j = 0; j < NUM_HAMSTERS; ++j) {
			hamsters[j]->wheels(-50, 50);
		}
		wait(500); // 0.5 seconds
	}

	// stop
	for(i = 0; i < NUM_HAMSTERS; ++i) {
		hamsters[i]->stop();
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example25: Mutiple Hand Follower
void example_multiple_hand_follower(void) {
	HamsterS* hamsters[NUM_HAMSTERS];
	int proximity, left_speed, right_speed;
	int i, j;

	for(i = 0; i < NUM_HAMSTERS; ++i) {
		hamsters[i] = hamster_s_create();
	}

	// wait until four robots are ready
	wait_until_ready();

	for(i = 0; i < 500; ++i) { // 10 seconds
		for(j = 0; j < NUM_HAMSTERS; ++j) {
			// left wheel
			proximity = hamsters[j]->left_proximity();
			if(proximity > 15) {
				left_speed = (40 - proximity) * 4;
			} else {
				left_speed = 0;
			}

			// right wheel
			proximity = hamsters[j]->right_proximity();
			if(proximity > 15) {
				right_speed = (40 - proximity) * 4;
			} else {
				right_speed = 0;
			}

			hamsters[j]->wheels(left_speed, right_speed);
		}
		wait(20); // 20 msec
	}

	// stop
	for(i = 0; i < NUM_HAMSTERS; ++i) {
		hamsters[i]->stop();
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example26: Advanced - Sensor
void sensor_execute(void* arg) {
	printf("Signal Strength: %d\n", hamster_s_signal_strength());
	printf("Proximity: %d, %d\n", hamster_s_left_proximity(), hamster_s_right_proximity());
	printf("Floor: %d, %d\n", hamster_s_left_floor(), hamster_s_right_floor());
	printf("Acceleration: %d, %d, %d\n", hamster_s_acceleration_x(), hamster_s_acceleration_y(), hamster_s_acceleration_z());
	printf("Light: %d\n", hamster_s_light());
	printf("Temperature: %d\n", hamster_s_temperature());
	printf("Input: %d, %d\n", hamster_s_input_a(), hamster_s_input_b());
	printf("Tilt: %d\n", hamster_s_tilt());
	printf("Battery State: %d\n\n", hamster_s_battery_state());
}

void example_advanced_sensor(void) {
	hamster_s_create();

	// set a periodic (20 msec) callback
	set_executable(sensor_execute, NULL);

	wait(10000); // 10 seconds

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example27: Advanced - Theremin Sound
void theremin_sound_execute(void* arg) {
	double* hz = (double*)arg;
	int proximity = hamster_s_left_proximity();
	if(proximity < 10) {
		proximity = 0;
	}
	*hz = (*hz * 5 + proximity * 50) / 10.0;
	hamster_s_buzzer(*hz);
}

void example_advanced_theremin_sound(void) {
	double hz = 0;

	hamster_s_create();

	// set a periodic (20 msec) callback
	set_executable(theremin_sound_execute, &hz);

	wait(10000); // 10 seconds
	hamster_s_buzzer(0); // clear note

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example28: Advanced - Simple Hand Follower
int calc_speed(int proximity) {
	return (proximity > 15) ? (40 - proximity) * 4 : 0;
}

void simple_hand_follower_execute(void* arg) {
	int left_speed = calc_speed(hamster_s_left_proximity());
	int right_speed = calc_speed(hamster_s_right_proximity());
	hamster_s_wheels(left_speed, right_speed);
}

void example_advanced_simple_hand_follower(void) {
	hamster_s_create();

	// set a periodic (20 msec) callback
	set_executable(simple_hand_follower_execute, NULL);

	wait(10000); // 10 seconds
	hamster_s_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example29: Advanced - Hand Detector
int hand_detector_evaluate(void* arg) {
	return (hamster_s_left_proximity() > 30) ? 1 : 0;
}

void example_advanced_hand_detector(void) {
	hamster_s_create();

	// wait until the value of the left proximity is greater than 30
	wait_until(hand_detector_evaluate, NULL);

	// move backward 5cm with beep sound
	hamster_s_sound("beep");
	hamster_s_move_backward(5, 60);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example30: Advanced - Extension - Button
// This example requires a Hamster's extension kit
void button_execute(void* arg) {
	if(hamster_s_input_b() == 0) {
		hamster_s_buzzer(1000);
	} else {
		hamster_s_buzzer(0);
	}
}

void example_advanced_extension_button(void) {
	hamster_s_create();

	// set a port B to the digital input mode
	hamster_s_io_mode_b("digital input");

	// set a periodic (20 msec) callback
	set_executable(button_execute, NULL);

	wait(10000); // 10 seconds

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example31: Advanced - Extension - Potentiometer
// This example requires a Hamster's extension kit
void potentiometer_execute(void* arg) {
    hamster_s_buzzer(hamster_s_input_a());
}

void example_advanced_extension_potentiometer(void) {
	hamster_s_create();

	// set a port A to the analog input mode
	hamster_s_io_mode_a("analog input");

	// set a periodic (20 msec) callback
	set_executable(potentiometer_execute, NULL);

	wait(10000); // 10 seconds

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example32: Advanced - Dual Theremin Sound
struct dual_theremin_sound {
	HamsterS* hamster1;
	HamsterS* hamster2;
	double hz1, hz2;
};

void dual_theremin_sound_execute(void* arg) {
	struct dual_theremin_sound* context = (struct dual_theremin_sound*)arg;

	int proximity = context->hamster1->left_proximity();
    if(proximity < 10) proximity = 0;
    context->hz1 = (context->hz1 * 5 + proximity * 50) / 10.0;
    context->hamster1->buzzer(context->hz1);

    proximity = context->hamster2->left_proximity();
    if(proximity < 10) proximity = 0;
    context->hz2 = (context->hz2 * 5 + proximity * 50) / 10.0;
    context->hamster2->buzzer(context->hz2);
}

void example_advanced_dual_theremin_sound(void) {
	struct dual_theremin_sound context;

	context.hamster1 = hamster_s_create();
	context.hamster2 = hamster_s_create();
	context.hz1 = 0;
	context.hz2 = 0;

	// wait until two robots are ready
	wait_until_ready();

	// set a periodic (20 msec) callback
	set_executable(dual_theremin_sound_execute, &context);

	wait(10000); // 10 seconds

	// clear sounds
	context.hamster1->buzzer(0);
	context.hamster2->buzzer(0);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example33: Advanced - Multiple Hand Follower
void multiple_hand_follower_execute(void* arg) {
	HamsterS** hamsters = (HamsterS**)arg;
	int left_speed, right_speed, i;

	for(i = 0; i < NUM_HAMSTERS; ++i) {
		left_speed = calc_speed(hamsters[i]->left_proximity());
		right_speed = calc_speed(hamsters[i]->right_proximity());
		hamsters[i]->wheels(left_speed, right_speed);
	}
}

void example_advanced_multiple_hand_follower(void) {
	HamsterS* hamsters[NUM_HAMSTERS];
	int i;

	for(i = 0; i < NUM_HAMSTERS; ++i) {
		hamsters[i] = hamster_s_create();
	}
	
	// wait until four robots are ready
	wait_until_ready();

	// set a periodic (20 msec) callback
	set_executable(multiple_hand_follower_execute, hamsters);

	wait(10000); // 10 seconds

	// stop
	for(i = 0; i < NUM_HAMSTERS; ++i) {
		hamsters[i]->stop();
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example34: Keyboard Controller
// Ctrl + Break to terminate
#define UP 72
#define DOWN 80
#define LEFT 75
#define RIGHT 77

void example_keyboard_controller(void) {
	hamster_s_create();

	while(1) {
	    switch(_getch()) {
	    	case UP: hamster_s_wheels(30, 30); break; // move forward
	    	case DOWN: hamster_s_wheels(-30, -30); break; // move backward
	    	case LEFT: hamster_s_wheels(-30, 30); break; // turn left
	    	case RIGHT: hamster_s_wheels(30, -30); break; // turn right
	    	case ' ': hamster_s_stop(); // stop
		}
		wait(10); // 10 msec
	}
}

// Example35: Keyboard Piano
// Ctrl + Break to terminate
void play(const char* pitch) {
	hamster_s_pitch("off"); // break for the same pitch
	wait(50);
    hamster_s_pitch(pitch);
}

void example_keyboard_piano(void) {
	hamster_s_create();
	
	while(1) {
	    switch(_getch()) {
	    	case ' ': hamster_s_pitch("off"); break;
	    	case 'a': play("C4"); break;
    		case 'w': play("C#4"); break;
    		case 's': play("D4"); break;
    		case 'e': play("Eb4"); break;
    		case 'd': play("E4"); break;
    		case 'f': play("F4"); break;
    		case 't': play("F#4"); break;
    		case 'g': play("G4"); break;
    		case 'y': play("G#4"); break;
    		case 'h': play("A4"); break;
    		case 'u': play("Bb4"); break;
    		case 'j': play("B4"); break;
    		case 'k': play("C5"); break;
    		case 'o': play("C#5"); break;
    		case 'l': play("D5"); break;
    		case 'p': play("Eb5"); break;
    		case ';': play("E5"); break;
    		case '\'': play("F5"); break;
		}
		wait(10); // 10 msec
	}
}

// Example36: Gripper
void example_gripper(void) {
	hamster_s_create();

	hamster_s_open_gripper();
	hamster_s_close_gripper();

	// turn off the power of a gripper
	hamster_s_release_gripper();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example37: Advanced - Multiple Hand Detector
int multiple_hand_detector_evaluate(void* arg) {
	return (((HamsterS*)arg)->left_proximity() > 30) ? 1 : 0;
}

void example_multiple_hand_detector(void) {
	HamsterS* hamster1 = hamster_s_create();
	HamsterS* hamster2 = hamster_s_create();

	// wait until the value of the left proximity is greater than 30
	wait_until(multiple_hand_detector_evaluate, hamster1);

	// move backward 5cm with beep sound
	hamster1->sound("beep");
	hamster1->move_backward(5, 60);

	// wait until the value of the left proximity is greater than 30
	wait_until(multiple_hand_detector_evaluate, hamster2);

	// move backward 5cm with beep sound
	hamster2->sound("beep");
	hamster2->move_backward(5, 60);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example38: Write Serial
void example_write_serial(void) {
	hamster_s_create();

	hamster_s_write_serial("a,", 2);
	wait(500);
	hamster_s_write_serial("b,", 2);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example39: Read Serial
void example_read_serial(void) {
	char text[100];

	hamster_s_create();

	while(1) {
		hamster_s_read_serial(text, 100);
		printf("%s\n", text);
		wait(20);
	}
}

int main(int argc, char *argv[]) {
//	example_scan();
	example_basic_movement1();
//	example_basic_movement2();
//	example_basic_movement3();
//	example_basic_movement4();
//	example_simple_movement();
//	example_acceleration();
//	example_led();
//	example_siren();
//	example_melody();
//	example_sound1();
//	example_sound2();
//	example_sensor();
//	example_staggering_walk();
//	example_theremin_sound();
//	example_simple_hand_follower();
//	example_builtin_line_tracer();
//	example_builtin_line_tracer_intersection();
//	example_hand_detector();
//	example_move_on_board();
//	example_extension_button();
//	example_extension_led_blink();
//	example_extension_potentiometer();
//	example_extension_smooth_led();
//	example_extension_servo();
//	example_extension_voltage();
//	example_dual_theremin_sound();
//	example_multiple_basic_movement();
//	example_multiple_hand_follower();
//	example_advanced_sensor();
//	example_advanced_theremin_sound();
//	example_advanced_simple_hand_follower();
//	example_advanced_hand_detector();
//	example_advanced_extension_button();
//	example_advanced_extension_potentiometer();
//	example_advanced_dual_theremin_sound();
//	example_advanced_multiple_hand_follower();
//	example_keyboard_controller();
//	example_keyboard_piano();
//	example_gripper();
//	example_multiple_hand_detector();
//	example_write_serial();
//	example_read_serial();

	return 0;
}
