// author: Kwang-Hyun Park (akaii@kw.ac.kr)

#include <stdio.h>
#include <conio.h>
#include "roboid.h"

#define NUM_TURTLES 4

// Example00: Scan
void example_scan(void) {
	scan(); // print serial ports
}

// Example01: Basic Movement
void example_basic_movement1(void) {
	turtle_create();

	// move forward/backward 6.5 cm, turn left/right 100.5 degrees with 60% velocity
	turtle_move_forward(6.5, 60);
	turtle_move_backward(6.5, 60);
	turtle_turn_left(100.5, 60);
	turtle_turn_right(100.5, 60);

	// move forward/backward, turn left/right for 1.5 seconds with 60% velocity
	turtle_move_forward_sec(1.5, 60);
	turtle_move_backward_sec(1.5, 60);
	turtle_turn_left_sec(1.5, 60);
	turtle_turn_right_sec(1.5, 60);

	// move forward/backward, turn left/right for 1.5 seconds with 60% velocity
	turtle_wheels(60, 60);
	wait(1500);
	turtle_wheels(-60, -60);
	wait(1500);
	turtle_wheels(-60, 60);
	wait(1500);
	turtle_wheels(60, -60);
	wait(1500);

	// stop
	turtle_stop();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example02: Basic Movement
void example_basic_movement2(void) {
	turtle_create();

	// pivot around left wheel 90/-90 degrees with 60% velocity
	turtle_pivot_left(90, 60);
	turtle_pivot_left(-90, 60);

	// pivot around right wheel 90/-90 degrees with 60% velocity
	turtle_pivot_right(90, 60);
	turtle_pivot_right(-90, 60);

	// pivot around left wheel 90 degrees with 60/-60% velocity
	turtle_pivot_left(90, 60);
	turtle_pivot_left(90, -60);

	// pivot around right wheel 90 degrees with 60/-60% velocity
	turtle_pivot_right(90, 60);
	turtle_pivot_right(90, -60);

	// pivot around left wheel for 1.5 seconds with 60/-60% velocity
	turtle_pivot_left_sec(1.5, 60);
	turtle_pivot_left_sec(1.5, -60);

	// pivot around right wheel for 1.5 seconds with 60/-60% velocity
	turtle_pivot_right_sec(1.5, 60);
	turtle_pivot_right_sec(1.5, -60);

	// pivot around left wheel for 1.5 seconds with 60/-60% velocity
	turtle_wheels(0, 60);
	wait(1500);
	turtle_wheels(0, -60);
	wait(1500);

	// pivot around right wheel for 1.5 seconds with 60/-60% velocity
	turtle_wheels(60, 0);
	wait(1500);
	turtle_wheels(-60, 0);
	wait(1500);

	// stop
	turtle_stop();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example03: Basic Movement
void example_basic_movement3(void) {
	turtle_create();

	// turn left 90/-90 degrees with radius 6cm with 60% velocity
	turtle_circle_left(90, 6, 60);
	turtle_circle_left(-90, 6, 60);

	// turn right 90/-90 degrees with radius 6cm with 60% velocity
	turtle_circle_right(90, 6, 60);
	turtle_circle_right(-90, 6, 60);

	// turn left 90 degrees with radius 6cm with 60/-60% velocity
	turtle_circle_left(90, 6, 60);
	turtle_circle_left(90, 6, -60);

	// turn right 90 degrees with radius 6cm with 60/-60% velocity
	turtle_circle_right(90, 6, 60);
	turtle_circle_right(90, 6, -60);

	// turn left for 1.5 seconds with radius 6cm with 60/-60% velocity
	turtle_circle_left_sec(1.5, 6, 60);
	turtle_circle_left_sec(1.5, 6, -60);

	// turn left for 1.5 seconds with radius 6cm with 60/-60% velocity
	turtle_circle_right_sec(1.5, 6, 60);
	turtle_circle_right_sec(1.5, 6, -60);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example04: Simple Movement
void example_simple_movement(void) {
	int i;

	turtle_create();

	for(i = 0; i < 10; ++i) {
		turtle_move_forward(6, 50); // move forward 6cm with 50% velocity
		turtle_move_backward(6, 50); // move backward 6cm with 50% velocity
		turtle_turn_left(90, 50); // turn left 90 degrees with 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example05: Acceleration
void example_acceleration(void) {
	int i;

	turtle_create();

	// acceleration
	for(i = 0; i < 100; ++i) {
		turtle_wheels(i, i);
		wait(20); // 20 msec
	}

	// 0.5 seconds
	wait(500);

	// deceleration
	for(i = 100; i >= 0; --i) {
		turtle_wheels(i, i);
		wait(20); // 20 msec
	}

	turtle_stop();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-1: Square
void example_square(void) {
	int i;

	turtle_create();

	for(i = 0; i < 4; ++i) {
		turtle_move_forward(6, 50); // 6cm, 50% velocity
		turtle_turn_right(90, 50); // 90 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-2: Rectangle
void example_rectangle(void) {
	int i;

	turtle_create();

	for(i = 0; i < 2; ++i) {
		turtle_move_forward(6, 50); // 6cm, 50% velocity
		turtle_turn_right(90, 50); // 90 degrees, 50% velocity
		turtle_move_forward(3, 50); // 3cm, 50% velocity
		turtle_turn_right(90, 50); // 90 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-3: Triangle
void example_triangle1(void) {
	int i;

	turtle_create();

	for(i = 0; i < 3; ++i) {
		turtle_move_forward(6, 50); // 6cm, 50% velocity
		turtle_turn_right(120, 50); // 120 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-4: Triangle
void example_triangle2(void) {
	int i;

	turtle_create();

	turtle_move_forward(6, 50); // 6cm, 50% velocity
	turtle_turn_right(60, 50); // 60 degrees, 50% velocity
	turtle_move_backward(6, 50); // 6cm, 50% velocity
	turtle_turn_right(60, 50); // 60 degrees, 50% velocity
	turtle_move_forward(6, 50); // 6cm, 50% velocity

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-5: Hexagon
void example_hexagon(void) {
	int i;

	turtle_create();

	for(i = 0; i < 6; ++i) {
		turtle_move_forward(6, 50); // 6cm, 50% velocity
		turtle_turn_right(60, 50); // 60 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-6: Leaf
void example_leaf(void) {
	int i;

	turtle_create();

	for(i = 0; i < 5; ++i) {
		turtle_pivot_right(180, 50); // 180 degrees, 50% velocity
		turtle_turn_right(108, 50); // 108 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-7: Star
void example_star1(void) {
	int i;

	turtle_create();

	for(i = 0; i < 5; ++i) {
		turtle_move_forward(6, 50); // 6cm, 50% velocity
		turtle_turn_right(144, 50); // 144 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-8: Star
void example_star2(void) {
	int i;

	turtle_create();

	for(i = 0; i < 6; ++i) {
		turtle_move_forward(3, 50); // 3cm, 50% velocity
		turtle_turn_right(60, 50); // 60 degrees, 50% velocity
		turtle_move_forward(3, 50); // 3cm, 50% velocity
		turtle_turn_left(120, 50); // 120 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-9: Flower
void example_flower(void) {
	int i;

	turtle_create();

	for(i = 0; i < 6; ++i) {
		turtle_pivot_right(360, 50); // 360 degrees, 50% velocity
		turtle_turn_right(60, 50); // 60 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example06-10: Concave Star
void example_concave_star(void) {
	int i;

	turtle_create();

	for(i = 0; i < 4; ++i) {
		turtle_pivot_right(90, 50); // 90 degrees, 50% velocity
		turtle_turn_right(180, 50); // 180 degrees, 50% velocity
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example07: Led
void example_led(void) {
	turtle_create();

	turtle_rgb(255, 0, 0); // red
	wait(500); // 0.5 seconds

	turtle_rgb(255, 63, 0); // orange
	wait(500); // 0.5 seconds

	turtle_rgb(255, 255, 0); // yellow
	wait(500); // 0.5 seconds

	turtle_rgb(0, 255, 0); // green
	wait(500); // 0.5 seconds

	turtle_rgb(0, 255, 255); // sky blue
	wait(500); // 0.5 seconds

	turtle_rgb(0, 0, 255); // blue
	wait(500); // 0.5 seconds

	turtle_rgb(63, 0, 255); // violet
	wait(500); // 0.5 seconds

	turtle_rgb(255, 0, 255); // purple
	wait(500); // 0.5 seconds

	turtle_rgb(255, 255, 255); // white
	wait(500); // 0.5 seconds

	turtle_led("red"); // red
	wait(500); // 0.5 seconds

	turtle_led("orange"); // orange
	wait(500); // 0.5 seconds

	turtle_led("yellow"); // yellow
	wait(500); // 0.5 seconds

	turtle_led("green"); // green
	wait(500); // 0.5 seconds

	turtle_led("sky blue"); // sky blue
	wait(500); // 0.5 seconds

	turtle_led("blue"); // blue
	wait(500); // 0.5 seconds

	turtle_led("violet"); // violet
	wait(500); // 0.5 seconds

	turtle_led("purple"); // purple
	wait(500); // 0.5 seconds

	turtle_led("white"); // white
	wait(500); // 0.5 seconds

	turtle_led("off"); // turn off

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example08: Siren
void example_siren(void) {
	int i, hz;

	turtle_create();

	for(i = 0; i < 10; ++i) {
		for(hz = 500; hz <= 700; hz += 10) {
			turtle_buzzer(hz);
			wait(20); // 20 msec
		}
	}
	turtle_buzzer(0); // clear buzzer

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example09: Melody
void example_melody(void) {
	int i, j;

	turtle_create();

	for(i = 0; i < 2; ++i) {
		for(j = 0; j < 2; ++j) {
			turtle_note("C4", 0.5);
			turtle_note("E4", 0.5);
			turtle_note("G4", 0.5);
		}

		for(j = 0; j < 3; ++j) {
			turtle_note("A4", 0.5);
		}

		turtle_note("G4", 1);
		turtle_note("off", 0.5);

		for(j = 0; j < 3; ++j) {
			turtle_note("F4", 0.5);
		}

		for(j = 0; j < 3; ++j) {
			turtle_note("E4", 0.5);
		}

		for(j = 0; j < 3; ++j) {
			turtle_note("D4", 0.5);
		}

		turtle_note("C4", 1);
		turtle_note("off", 0.5);

		turtle_tempo(120); // speed up
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example10: Sound
void example_sound1(void) {
	turtle_create();

	// play sound 2 times
	turtle_repeat_sound("beep", 2);
	wait(1000);

	turtle_sound("random beep");
	wait(1000);

	turtle_sound("siren");
	wait(1000);

	turtle_sound("engine");
	wait(1000);

	turtle_sound("robot");
	wait(1000);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example11: Sound
void example_sound2(void) {
	turtle_create();

	// play sound and wait until done
	turtle_sound_until_done("siren");

	// random beep: 3 times
	turtle_repeat_sound_until_done("random beep", 3);

	turtle_sound_until_done("dibidibidip");

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example12: Sensor
void example_sensor(void) {
	int i;

	turtle_create();

	for(i = 0; i < 500; ++i) { // 10 seconds
		printf("Signal Strength: %d\n", turtle_signal_strength());
		printf("Floor: %d\n", turtle_floor());
		printf("Acceleration: %d, %d, %d\n", turtle_acceleration_x(), turtle_acceleration_y(), turtle_acceleration_z());
		printf("Temperature: %d\n", turtle_temperature());
		printf("Button: %d\n", turtle_button());
		printf("Clicked: %d\n", turtle_clicked());
		printf("Double Clicked: %d\n", turtle_double_clicked());
		printf("Long Pressed: %d\n", turtle_long_pressed());
		printf("Color Number: %d\n", turtle_color_number());
		printf("Color Pattern: %d\n", turtle_color_pattern());
		printf("Tilt: %d\n", turtle_tilt());
		printf("Battery State: %d\n\n", turtle_battery_state());

		wait(20); // 20 msec
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example13: Staggering Walk
void example_staggering_walk(void) {
	int acc, count;

	turtle_create();

	for(count = 10; count > 0; ) {
		turtle_wheels(-100, -100); // move backward
		acc = turtle_acceleration_x();
		if(acc > 2000 || acc < -2000) {
			turtle_wheels(100, 100); // move forward
			wait(250); // 0.25 seconds

			turtle_wheels(-50, 50); // turn left
			wait(500); // 0.5 seconds

			-- count;
		}
		wait(10); // 10 msec
	}
	turtle_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example14: Theremin Sound
void example_theremin_sound(void) {
	double hz = 0;
	int floor, i;

	turtle_create();

	for(i = 0; i < 500; ++i) { // 10 seconds
		floor = turtle_floor();
		if(floor < 10) {
			floor = 0;
		}
		hz = (hz * 5 + floor * 50) / 10.0;
		turtle_buzzer(hz);

		wait(20); // 20 msec
	}
	turtle_buzzer(0); // clear sound

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example15: Builtin Line Tracer
void example_builtin_line_tracer(void) {
	turtle_create();

	// follow black line
	turtle_black_line();
	wait(5000); // 5 seconds

	turtle_line_speed(8); // speed up
	wait(2000); // 5 seconds

	turtle_line_speed(5); // speed down
	wait(2000); // 5 seconds

	// follow red line
	turtle_red_line();
	wait(5000); // 5 seconds

	// follow green line
	turtle_green_line();
	wait(5000); // 5 seconds

	// follow blue line
	turtle_blue_line();
	wait(5000); // 5 seconds

	// follow any line
	turtle_any_line();
	wait(5000); // 5 seconds

	turtle_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example16: Builtin Line Tracer Intersection
void example_builtin_line_tracer_intersection(void) {
	turtle_create();

	// follow black line until red
	turtle_black_line_until_red();

	// cross at intersection
	turtle_cross_forward();

	// follow black line until red
	turtle_black_line_until_red();

	// turn left at intersection
	turtle_cross_left();

	// follow black line until red
	turtle_black_line_until_red();
	
	// turn right at intersection
	turtle_cross_right();
	
	// follow black line until red
	turtle_black_line_until_red();
	
	// u-turn at intersection
	turtle_cross_uturn();
	
	// follow black line until red
	turtle_black_line_until_red();

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example17: Color Detector
void example_color_detector(void) {
	turtle_create();

	// wait while the color is not red
	while(turtle_is_color("red") == 0) {
		wait(10); // 10 msec
	}

	turtle_sound("beep"); // beep
	turtle_wheels(-50, -50); // move backward

	// wait while the color is not blue
	while(turtle_is_color("blue") == 0) {
		wait(10); // 10 msec
	}

	turtle_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example18: Color Pattern Detector
void example_color_pattern_detector(void) {
	turtle_create();

	// wait while the color pattern is not (purple, red)
	while(turtle_is_color_pattern("purple", "red") == 0) {
		wait(10); // 10 msec
	}

	turtle_sound("beep"); // beep
	turtle_move_backward(6, 50); // move backward 6cm with 50% velocity

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example19: Dual Theremin Sound
void example_dual_theremin_sound(void) {
	double hz1 = 0, hz2 = 0;
	int floor, i;

	Turtle* turtle1 = turtle_create();
	Turtle* turtle2 = turtle_create();

	// wait until two robots are ready
	wait_until_ready();

	for(i = 0; i < 500; ++i) { // 10 seconds
		floor = turtle1->floor();
		if(floor < 10) floor = 0;
		hz1 = (hz1 * 5 + floor * 50) / 10.0;
		turtle1->buzzer(hz1);

		floor = turtle2->floor();
		if(floor < 10) floor = 0;
		hz2 = (hz2 * 5 + floor * 50) / 10.0;
		turtle2->buzzer(hz2);

		wait(20); // 20 msec
	}

	// clear sounds
	turtle1->buzzer(0);
	turtle2->buzzer(0);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example20: Mutiple Simple Movement
void example_multiple_basic_movement(void) {
	Turtle* turtles[NUM_TURTLES];
	int i, j;

	for(i = 0; i < NUM_TURTLES; ++i) {
		turtles[i] = turtle_create();
	}

	// wait until four robots are ready
	wait_until_ready();

	for(i = 0; i < 10; ++i) {
		// move forward
		for(j = 0; j < NUM_TURTLES; ++j) {
			turtles[j]->wheels(50, 50);
		}
		wait(500); // 0.5 seconds

		// move backward
		for(j = 0; j < NUM_TURTLES; ++j) {
			turtles[j]->wheels(-50, -50);
		}
		wait(500); // 0.5 seconds

		// turn left
		for(j = 0; j < NUM_TURTLES; ++j) {
			turtles[j]->wheels(-50, 50);
		}
		wait(500); // 0.5 seconds
	}

	// stop
	for(i = 0; i < NUM_TURTLES; ++i) {
		turtles[i]->stop();
	}

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example21: Advanced - Sensor
void sensor_execute(void* arg) {
	printf("Signal Strength: %d\n", turtle_signal_strength());
	printf("Floor: %d\n", turtle_floor());
	printf("Acceleration: %d, %d, %d\n", turtle_acceleration_x(), turtle_acceleration_y(), turtle_acceleration_z());
	printf("Temperature: %d\n", turtle_temperature());
	printf("Button: %d\n", turtle_button());
	printf("Clicked: %d\n", turtle_clicked());
	printf("Double Clicked: %d\n", turtle_double_clicked());
	printf("Long Pressed: %d\n", turtle_long_pressed());
	printf("Color Number: %d\n", turtle_color_number());
	printf("Color Pattern: %d\n", turtle_color_pattern());
	printf("Tilt: %d\n", turtle_tilt());
	printf("Battery State: %d\n\n", turtle_battery_state());
}

void example_advanced_sensor(void) {
	turtle_create();

	// set a periodic (20 msec) callback
	set_executable(sensor_execute, NULL);

	wait(10000); // 10 seconds

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example22: Advanced - Theremin Sound
void theremin_sound_execute(void* arg) {
	double* hz = (double*)arg;
	int floor = turtle_floor();
	if(floor < 10) {
		floor = 0;
	}
	*hz = (*hz * 5 + floor * 50) / 10.0;
	turtle_buzzer(*hz);
}

void example_advanced_theremin_sound(void) {
	double hz = 0;

	turtle_create();

	// set a periodic (20 msec) callback
	set_executable(theremin_sound_execute, &hz);

	wait(10000); // 10 seconds
	turtle_buzzer(0); // clear note

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example23: Advanced - Color Detector
int color_detector_evaluate_red(void* arg) {
	return turtle_is_color("red");
}

int color_detector_evaluate_blue(void* arg) {
	return turtle_is_color("blue");
}

void example_advanced_color_detector(void) {
	turtle_create();

	// wait until the color is red
	wait_until(color_detector_evaluate_red, NULL);

	turtle_sound("beep"); // beep
	turtle_wheels(-50, -50); // move backward

	// wait until the color is blue
	wait_until(color_detector_evaluate_blue, NULL);

	turtle_stop(); // stop

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example24: Advanced - Color Pattern Detector
int color_pattern_detector_evaluate(void* arg) {
	return turtle_is_color_pattern("purple", "red");
}

void example_advanced_color_pattern_detector(void) {
	turtle_create();

	// wait until the color pattern is (purple, red)
	wait_until(color_pattern_detector_evaluate, NULL);

	turtle_sound("beep"); // beep
	turtle_move_backward(6, 50); // move backward 6cm with 50% velocity

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example25: Advanced - Dual Theremin Sound
struct dual_theremin_sound {
	Turtle* turtle1;
	Turtle* turtle2;
	double hz1, hz2;
};

void dual_theremin_sound_execute(void* arg) {
	struct dual_theremin_sound* context = (struct dual_theremin_sound*)arg;

	int floor = context->turtle1->floor();
    if(floor < 10) floor = 0;
    context->hz1 = (context->hz1 * 5 + floor * 50) / 10.0;
    context->turtle1->buzzer(context->hz1);

    floor = context->turtle2->floor();
    if(floor < 10) floor = 0;
    context->hz2 = (context->hz2 * 5 + floor * 50) / 10.0;
    context->turtle2->buzzer(context->hz2);
}

void example_advanced_dual_theremin_sound(void) {
	struct dual_theremin_sound context;

	context.turtle1 = turtle_create();
	context.turtle2 = turtle_create();
	context.hz1 = 0;
	context.hz2 = 0;

	// wait until two robots are ready
	wait_until_ready();

	// set a periodic (20 msec) callback
	set_executable(dual_theremin_sound_execute, &context);

	wait(10000); // 10 seconds

	// clear sounds
	context.turtle1->buzzer(0);
	context.turtle2->buzzer(0);

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example26: Keyboard Controller
// Ctrl + Break to terminate
#define UP 72
#define DOWN 80
#define LEFT 75
#define RIGHT 77

void example_keyboard_controller(void) {
	turtle_create();

	while(1) {
	    switch(_getch()) {
	    	case UP: turtle_wheels(50, 50); break; // move forward
	    	case DOWN: turtle_wheels(-50, -50); break; // move backward
	    	case LEFT: turtle_wheels(-50, 50); break; // turn left
	    	case RIGHT: turtle_wheels(50, -50); break; // turn right
	    	case ' ': turtle_stop(); // stop
		}
		wait(10); // 10 msec
	}
}

// Example27: Keyboard Piano
// Ctrl + Break to terminate
void play(const char* pitch) {
	turtle_pitch("off"); // break for the same pitch
	wait(50);
    turtle_pitch(pitch);
}

void example_keyboard_piano(void) {
	turtle_create();
	
	while(1) {
	    switch(_getch()) {
	    	case ' ': turtle_pitch("off"); break;
	    	case 'a': play("C4"); break;
    		case 'w': play("C#4"); break;
    		case 's': play("D4"); break;
    		case 'e': play("Eb4"); break;
    		case 'd': play("E4"); break;
    		case 'f': play("F4"); break;
    		case 't': play("F#4"); break;
    		case 'g': play("G4"); break;
    		case 'y': play("G#4"); break;
    		case 'h': play("A4"); break;
    		case 'u': play("Bb4"); break;
    		case 'j': play("B4"); break;
    		case 'k': play("C5"); break;
    		case 'o': play("C#5"); break;
    		case 'l': play("D5"); break;
    		case 'p': play("Eb5"); break;
    		case ';': play("E5"); break;
    		case '\'': play("F5"); break;
		}
		wait(10); // 10 msec
	}
}

// Example28: Advanced - Multiple Color Detector
int multiple_color_detector_evaluate(void* arg) {
	return ((Turtle*)arg)->is_color("red");
}

void example_multiple_color_detector(void) {
	Turtle* turtle1 = turtle_create();
	Turtle* turtle2 = turtle_create();

	// wait until the color is red
	wait_until(multiple_color_detector_evaluate, turtle1);

	turtle1->sound("beep"); // beep
	turtle1->move_backward(6, 50); // move backward 6cm with 50% velocity

	// wait until the color is red
	wait_until(multiple_color_detector_evaluate, turtle2);

	turtle2->sound("beep"); // beep
	turtle2->move_backward(6, 50); // move backward 6cm with 50% velocity

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

// Example29: Advanced - Multiple Color Pattern Detector
int multiple_color_pattern_detector_evaluate(void* arg) {
	return ((Turtle*)arg)->is_color_pattern("purple", "red");
}

void example_multiple_color_detector_detector(void) {
	Turtle* turtle1 = turtle_create();
	Turtle* turtle2 = turtle_create();

	// wait until the color pattern is (purple, red)
	wait_until(multiple_color_pattern_detector_evaluate, turtle1);

	turtle1->sound("beep"); // beep
	turtle1->move_backward(6, 50); // move backward 6cm with 50% velocity

	// wait until the color pattern is (purple, red)
	wait_until(multiple_color_pattern_detector_evaluate, turtle2);

	turtle2->sound("beep"); // beep
	turtle2->move_backward(6, 50); // move backward 6cm with 50% velocity

	// don't forget to dispose
	dispose_all(); // disconnect and release memory
}

int main(int argc, char *argv[]) {
//	example_scan();
	example_basic_movement1();
//	example_basic_movement2();
//	example_basic_movement3();
//	example_simple_movement();
//	example_acceleration();
//	example_square();
//	example_rectangle();
//	example_triangle1();
//	example_triangle2();
//	example_hexagon();
//	example_leaf();
//	example_star1();
//	example_star2();
//	example_flower();
//	example_concave_star();
//	example_led();
//	example_siren();
//	example_melody();
//	example_sound1();
//	example_sound2();
//	example_sensor();
//	example_staggering_walk();
//	example_theremin_sound();
//	example_builtin_line_tracer();
//	example_builtin_line_tracer_intersection();
//	example_color_detector();
//	example_color_pattern_detector();
//	example_dual_theremin_sound();
//	example_multiple_basic_movement();
//	example_advanced_sensor();
//	example_advanced_theremin_sound();
//	example_advanced_color_detector();
//	example_advanced_color_pattern_detector();
//	example_advanced_dual_theremin_sound();
//	example_keyboard_controller();
//	example_keyboard_piano();
//	example_multiple_color_detector();
//	example_multiple_color_detector_detector();

	return 0;
}
