/*
 * Part of the ROBOID project - http://hamster.school
 * Copyright (C) 2016 Kwang-Hyun Park (akaii@kw.ac.kr)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
*/

#ifndef _ROBOID_H_
#define _ROBOID_H_

#define DATA_TYPE_INTEGER 4
#define DATA_TYPE_FLOAT 5

#define DEVICE_TYPE_SENSOR 0
#define DEVICE_TYPE_EFFECTOR 1
#define DEVICE_TYPE_EVENT 2
#define DEVICE_TYPE_COMMAND 3

#define HAMSTER_ID "kr.robomation.physical.hamster"

#define HAMSTER_LEFT_WHEEL 0x00400000
#define HAMSTER_RIGHT_WHEEL 0x00400001
#define HAMSTER_BUZZER 0x00400002
#define HAMSTER_OUTPUT_A 0x00400003
#define HAMSTER_OUTPUT_B 0x00400004
#define HAMSTER_TOPOLOGY 0x00400005
#define HAMSTER_LEFT_LED 0x00400006
#define HAMSTER_RIGHT_LED 0x00400007
#define HAMSTER_NOTE 0x00400008
#define HAMSTER_LINE_TRACER_MODE 0x00400009
#define HAMSTER_LINE_TRACER_SPEED 0x0040000a
#define HAMSTER_IO_MODE_A 0x0040000b
#define HAMSTER_IO_MODE_B 0x0040000c
#define HAMSTER_CONFIG_PROXIMITY 0x0040000d
#define HAMSTER_CONFIG_GRAVITY 0x0040000e
#define HAMSTER_CONFIG_BAND_WIDTH 0x0040000f

#define HAMSTER_SIGNAL_STRENGTH 0x00400010
#define HAMSTER_LEFT_PROXIMITY 0x00400011
#define HAMSTER_RIGHT_PROXIMITY 0x00400012
#define HAMSTER_LEFT_FLOOR 0x00400013
#define HAMSTER_RIGHT_FLOOR 0x00400014
#define HAMSTER_ACCELERATION 0x00400015
#define HAMSTER_LIGHT 0x00400016
#define HAMSTER_TEMPERATURE 0x00400017
#define HAMSTER_INPUT_A 0x00400018
#define HAMSTER_INPUT_B 0x00400019
#define HAMSTER_LINE_TRACER_STATE 0x0040001a
#define HAMSTER_TILT 0x0040001b
#define HAMSTER_BATTERY_STATE 0x0040001c

#define HAMSTER_TOPOLOGY_NONE 0
#define HAMSTER_TOPOLOGY_DAISY_CHAIN 1
#define HAMSTER_TOPOLOGY_STAR 2
#define HAMSTER_TOPOLOGY_EXTENDED_STAR 3

#define HAMSTER_LED_OFF "OFF"
#define HAMSTER_LED_RED "RED"
#define HAMSTER_LED_YELLOW "YELLOW"
#define HAMSTER_LED_GREEN "GREEN"
#define HAMSTER_LED_CYAN "CYAN"
#define HAMSTER_LED_SKY_BLUE "SKY BLUE"
#define HAMSTER_LED_BLUE "BLUE"
#define HAMSTER_LED_MAGENTA "MAGENTA"
#define HAMSTER_LED_PURPLE "PURPLE"
#define HAMSTER_LED_WHITE "WHITE"

#define HAMSTER_LINE_TRACER_MODE_OFF 0
#define HAMSTER_LINE_TRACER_MODE_BLACK_LEFT_SENSOR 1
#define HAMSTER_LINE_TRACER_MODE_BLACK_RIGHT_SENSOR 2
#define HAMSTER_LINE_TRACER_MODE_BLACK_BOTH_SENSORS 3
#define HAMSTER_LINE_TRACER_MODE_BLACK_TURN_LEFT 4
#define HAMSTER_LINE_TRACER_MODE_BLACK_TURN_RIGHT 5
#define HAMSTER_LINE_TRACER_MODE_BLACK_MOVE_FORWARD 6
#define HAMSTER_LINE_TRACER_MODE_BLACK_UTURN 7
#define HAMSTER_LINE_TRACER_MODE_WHITE_LEFT_SENSOR 8
#define HAMSTER_LINE_TRACER_MODE_WHITE_RIGHT_SENSOR 9
#define HAMSTER_LINE_TRACER_MODE_WHITE_BOTH_SENSORS 10
#define HAMSTER_LINE_TRACER_MODE_WHITE_TURN_LEFT 11
#define HAMSTER_LINE_TRACER_MODE_WHITE_TURN_RIGHT 12
#define HAMSTER_LINE_TRACER_MODE_WHITE_MOVE_FORWARD 13
#define HAMSTER_LINE_TRACER_MODE_WHITE_UTURN 14

#define HAMSTER_IO_MODE_ANALOG_INPUT "ANALOG INPUT"
#define HAMSTER_IO_MODE_DIGITAL_INPUT "DIGITAL INPUT"
#define HAMSTER_IO_MODE_SERVO_OUTPUT "SERVO OUTPUT"
#define HAMSTER_IO_MODE_PWM_OUTPUT "PWM OUTPUT"
#define HAMSTER_IO_MODE_DIGITAL_OUTPUT "DIGITAL OUTPUT"

#define HAMSTER_NOTE_OFF "OFF"
#define HAMSTER_NOTE_A_0 "A0"
#define HAMSTER_NOTE_A_SHARP_0 "A#0"
#define HAMSTER_NOTE_B_FLAT_0 "Bb0"
#define HAMSTER_NOTE_B_0 "B0"
#define HAMSTER_NOTE_C_1 "C1"
#define HAMSTER_NOTE_C_SHARP_1 "C#1"
#define HAMSTER_NOTE_D_FLAT_1 "Db1"
#define HAMSTER_NOTE_D_1 "D1"
#define HAMSTER_NOTE_D_SHARP_1 "D#1"
#define HAMSTER_NOTE_E_FLAT_1 "Eb1"
#define HAMSTER_NOTE_E_1 "E1"
#define HAMSTER_NOTE_F_1 "F1"
#define HAMSTER_NOTE_F_SHARP_1 "F#1"
#define HAMSTER_NOTE_G_FLAT_1 "Gb1"
#define HAMSTER_NOTE_G_1 "G1"
#define HAMSTER_NOTE_G_SHARP_1 "G#1"
#define HAMSTER_NOTE_A_FLAT_1 "Ab1"
#define HAMSTER_NOTE_A_1 "A1"
#define HAMSTER_NOTE_A_SHARP_1 "A#1"
#define HAMSTER_NOTE_B_FLAT_1 "Bb1"
#define HAMSTER_NOTE_B_1 "B1"
#define HAMSTER_NOTE_C_2 "C2"
#define HAMSTER_NOTE_C_SHARP_2 "C#2"
#define HAMSTER_NOTE_D_FLAT_2 "Db2"
#define HAMSTER_NOTE_D_2 "D2"
#define HAMSTER_NOTE_D_SHARP_2 "D#2"
#define HAMSTER_NOTE_E_FLAT_2 "Eb2"
#define HAMSTER_NOTE_E_2 "E2"
#define HAMSTER_NOTE_F_2 "F2"
#define HAMSTER_NOTE_F_SHARP_2 "F#2"
#define HAMSTER_NOTE_G_FLAT_2 "Gb2"
#define HAMSTER_NOTE_G_2 "G2"
#define HAMSTER_NOTE_G_SHARP_2 "G#2"
#define HAMSTER_NOTE_A_FLAT_2 "Ab2"
#define HAMSTER_NOTE_A_2 "A2"
#define HAMSTER_NOTE_A_SHARP_2 "A#2"
#define HAMSTER_NOTE_B_FLAT_2 "Bb2"
#define HAMSTER_NOTE_B_2 "B2"
#define HAMSTER_NOTE_C_3 "C3"
#define HAMSTER_NOTE_C_SHARP_3 "C#3"
#define HAMSTER_NOTE_D_FLAT_3 "Db3"
#define HAMSTER_NOTE_D_3 "D3"
#define HAMSTER_NOTE_D_SHARP_3 "D#3"
#define HAMSTER_NOTE_E_FLAT_3 "Eb3"
#define HAMSTER_NOTE_E_3 "E3"
#define HAMSTER_NOTE_F_3 "F3"
#define HAMSTER_NOTE_F_SHARP_3 "F#3"
#define HAMSTER_NOTE_G_FLAT_3 "Gb3"
#define HAMSTER_NOTE_G_3 "G3"
#define HAMSTER_NOTE_G_SHARP_3 "G#3"
#define HAMSTER_NOTE_A_FLAT_3 "Ab3"
#define HAMSTER_NOTE_A_3 "A3"
#define HAMSTER_NOTE_A_SHARP_3 "A#3"
#define HAMSTER_NOTE_B_FLAT_3 "Bb3"
#define HAMSTER_NOTE_B_3 "B3"
#define HAMSTER_NOTE_C_4 "C4"
#define HAMSTER_NOTE_C_SHARP_4 "C#4"
#define HAMSTER_NOTE_D_FLAT_4 "Db4"
#define HAMSTER_NOTE_D_4 "D4"
#define HAMSTER_NOTE_D_SHARP_4 "D#4"
#define HAMSTER_NOTE_E_FLAT_4 "Eb4"
#define HAMSTER_NOTE_E_4 "E4"
#define HAMSTER_NOTE_F_4 "F4"
#define HAMSTER_NOTE_F_SHARP_4 "F#4"
#define HAMSTER_NOTE_G_FLAT_4 "Gb4"
#define HAMSTER_NOTE_G_4 "G4"
#define HAMSTER_NOTE_G_SHARP_4 "G#4"
#define HAMSTER_NOTE_A_FLAT_4 "Ab4"
#define HAMSTER_NOTE_A_4 "A4"
#define HAMSTER_NOTE_A_SHARP_4 "A#4"
#define HAMSTER_NOTE_B_FLAT_4 "Bb4"
#define HAMSTER_NOTE_B_4 "B4"
#define HAMSTER_NOTE_C_5 "C5"
#define HAMSTER_NOTE_C_SHARP_5 "C#5"
#define HAMSTER_NOTE_D_FLAT_5 "Db5"
#define HAMSTER_NOTE_D_5 "D5"
#define HAMSTER_NOTE_D_SHARP_5 "D#5"
#define HAMSTER_NOTE_E_FLAT_5 "Eb5"
#define HAMSTER_NOTE_E_5 "E5"
#define HAMSTER_NOTE_F_5 "F5"
#define HAMSTER_NOTE_F_SHARP_5 "F#5"
#define HAMSTER_NOTE_G_FLAT_5 "Gb5"
#define HAMSTER_NOTE_G_5 "G5"
#define HAMSTER_NOTE_G_SHARP_5 "G#5"
#define HAMSTER_NOTE_A_FLAT_5 "Ab5"
#define HAMSTER_NOTE_A_5 "A5"
#define HAMSTER_NOTE_A_SHARP_5 "A#5"
#define HAMSTER_NOTE_B_FLAT_5 "Bb5"
#define HAMSTER_NOTE_B_5 "B5"
#define HAMSTER_NOTE_C_6 "C6"
#define HAMSTER_NOTE_C_SHARP_6 "C#6"
#define HAMSTER_NOTE_D_FLAT_6 "Db6"
#define HAMSTER_NOTE_D_6 "D6"
#define HAMSTER_NOTE_D_SHARP_6 "D#6"
#define HAMSTER_NOTE_E_FLAT_6 "Eb6"
#define HAMSTER_NOTE_E_6 "E6"
#define HAMSTER_NOTE_F_6 "F6"
#define HAMSTER_NOTE_F_SHARP_6 "F#6"
#define HAMSTER_NOTE_G_FLAT_6 "Gb6"
#define HAMSTER_NOTE_G_6 "G6"
#define HAMSTER_NOTE_G_SHARP_6 "G#6"
#define HAMSTER_NOTE_A_FLAT_6 "Ab6"
#define HAMSTER_NOTE_A_6 "A6"
#define HAMSTER_NOTE_A_SHARP_6 "A#6"
#define HAMSTER_NOTE_B_FLAT_6 "Bb6"
#define HAMSTER_NOTE_B_6 "B6"
#define HAMSTER_NOTE_C_7 "C7"
#define HAMSTER_NOTE_C_SHARP_7 "C#7"
#define HAMSTER_NOTE_D_FLAT_7 "Db7"
#define HAMSTER_NOTE_D_7 "D7"
#define HAMSTER_NOTE_D_SHARP_7 "D#7"
#define HAMSTER_NOTE_E_FLAT_7 "Eb7"
#define HAMSTER_NOTE_E_7 "E7"
#define HAMSTER_NOTE_F_7 "F7"
#define HAMSTER_NOTE_F_SHARP_7 "F#7"
#define HAMSTER_NOTE_G_FLAT_7 "Gb7"
#define HAMSTER_NOTE_G_7 "G7"
#define HAMSTER_NOTE_G_SHARP_7 "G#7"
#define HAMSTER_NOTE_A_FLAT_7 "Ab7"
#define HAMSTER_NOTE_A_7 "A7"
#define HAMSTER_NOTE_A_SHARP_7 "A#7"
#define HAMSTER_NOTE_B_FLAT_7 "Bb7"
#define HAMSTER_NOTE_B_7 "B7"
#define HAMSTER_NOTE_C_8 "C8"

#define HAMSTER_NOTE_C "C"
#define HAMSTER_NOTE_C_SHARP "C#"
#define HAMSTER_NOTE_D_FLAT "Db"
#define HAMSTER_NOTE_D "D"
#define HAMSTER_NOTE_D_SHARP "D#"
#define HAMSTER_NOTE_E_FLAT "Eb"
#define HAMSTER_NOTE_E "E"
#define HAMSTER_NOTE_F "F"
#define HAMSTER_NOTE_F_SHARP "F#"
#define HAMSTER_NOTE_G_FLAT "Gb"
#define HAMSTER_NOTE_G "G"
#define HAMSTER_NOTE_G_SHARP "G#"
#define HAMSTER_NOTE_A_FLAT "Ab"
#define HAMSTER_NOTE_A "A"
#define HAMSTER_NOTE_A_SHARP "A#"
#define HAMSTER_NOTE_B_FLAT "Bb"
#define HAMSTER_NOTE_B "B"

#define HAMSTER_TILT_FORWARD 1
#define HAMSTER_TILT_BACKWARD -1
#define HAMSTER_TILT_LEFT 2
#define HAMSTER_TILT_RIGHT -2
#define HAMSTER_TILT_FLIP 3
#define HAMSTER_TILT_NOT -3

#define HAMSTER_BATTERY_NORMAL 2
#define HAMSTER_BATTERY_LOW 1
#define HAMSTER_BATTERY_EMPTY 0

typedef struct hamster {
	const char* (*get_name)(void);
	void (*set_name)(const char* name);
	const char* (*get_id)(void);
	int (*get_index)(void);
	int (*e)(int device_id);
	int (*read)(int device_id);
	int (*read_at)(int device_id, int index);
	int (*read_array)(int device_id, int* data, int length);
	float (*read_float)(int device_id);
	float (*read_float_at)(int device_id, int index);
	int (*read_float_array)(int device_id, float* data, int length);
	int (*write_empty)(int device_id);
	int (*write)(int device_id, int data);
	int (*write_at)(int device_id, int index, int data);
	int (*write_array)(int device_id, const int* data, int length);
	int (*write_float)(int device_id, float data);
	int (*write_float_at)(int device_id, int index, float data);
	int (*write_float_array)(int device_id, const float* data, int length);
	void (*reset)(void);
	void (*dispose)(void);
	void (*wheels)(double left_velocity, double right_velocity);
	void (*left_wheel)(double velocity);
	void (*right_wheel)(double velocity);
	void (*stop)(void);
	void (*move_forward)(double sec, double velocity);
	void (*move_backward)(double sec, double velocity);
	void (*turn_left)(double sec, double velocity);
	void (*turn_right)(double sec, double velocity);
	void (*line_tracer_mode)(int mode);
	void (*line_left)(void);
	void (*line_right)(void);
	void (*line_both)(void);
	void (*cross_forward)(void);
	void (*cross_left)(void);
	void (*cross_right)(void);
	void (*cross_uturn)(void);
	void (*white_line_left)(void);
	void (*white_line_right)(void);
	void (*white_line_both)(void);
	void (*white_cross_forward)(void);
	void (*white_cross_left)(void);
	void (*white_cross_right)(void);
	void (*white_cross_uturn)(void);
	void (*line_speed)(double speed);
	void (*line_tracer_speed)(double speed);
	void (*board_forward)(void);
	void (*board_left)(void);
	void (*board_right)(void);
	void (*leds)(const char* left_color, const char* right_color);
	void (*left_led)(const char* color);
	void (*right_led)(const char* color);
	void (*beep)(void);
	void (*buzzer)(double hz);
	void (*tempo)(double bpm);
	void (*pitch)(const char* pitch);
	void (*note)(const char* pitch, double beats);
	void (*io_mode_a)(const char* mode);
	void (*io_mode_b)(const char* mode);
	void (*output_a)(double value);
	void (*output_b)(double value);
	void (*open_gripper)(void);
	void (*close_gripper)(void);
	void (*release_gripper)(void);
	int (*signal_strength)(void);
	int (*left_proximity)(void);
	int (*right_proximity)(void);
	int (*left_floor)(void);
	int (*right_floor)(void);
	int (*acceleration_x)(void);
	int (*acceleration_y)(void);
	int (*acceleration_z)(void);
	int (*light)(void);
	int (*temperature)(void);
	int (*input_a)(void);
	int (*input_b)(void);
	int (*tilt)(void);
	int (*battery_state)(void);
} Hamster;

void scan(void);
void set_executable(void (*execute)(void* arg), void* arg);
void wait(int milliseconds);
void wait_until(int (*condition)(void* arg), void* arg);
void wait_until_ready(void);
void dispose_all(void);

Hamster* hamster_create(void);
Hamster* hamster_create_port(const char* port_name);
const char* hamster_get_name(void);
void hamster_set_name(const char* name);
const char* hamster_get_id(void);
int hamster_e(int device_id);
int hamster_read(int device_id);
int hamster_read_at(int device_id, int index);
int hamster_read_array(int device_id, int* data, int length);
float hamster_read_float(int device_id);
float hamster_read_float_at(int device_id, int index);
int hamster_read_float_array(int device_id, float* data, int length);
int hamster_write_empty(int device_id);
int hamster_write(int device_id, int data);
int hamster_write_at(int device_id, int index, int data);
int hamster_write_array(int device_id, const int* data, int length);
int hamster_write_float(int device_id, float data);
int hamster_write_float_at(int device_id, int index, float data);
int hamster_write_float_array(int device_id, const float* data, int length);
void hamster_reset(void);
void hamster_dispose(void);
void hamster_wheels(double left_velocity, double right_velocity);
void hamster_left_wheel(double velocity);
void hamster_right_wheel(double velocity);
void hamster_stop(void);
void hamster_move_forward(double sec, double velocity);
void hamster_move_backward(double sec, double velocity);
void hamster_turn_left(double sec, double velocity);
void hamster_turn_right(double sec, double velocity);
void hamster_line_tracer_mode(int mode);
void hamster_line_left(void);
void hamster_line_right(void);
void hamster_line_both(void);
void hamster_cross_forward(void);
void hamster_cross_left(void);
void hamster_cross_right(void);
void hamster_cross_uturn(void);
void hamster_white_line_left(void);
void hamster_white_line_right(void);
void hamster_white_line_both(void);
void hamster_white_cross_forward(void);
void hamster_white_cross_left(void);
void hamster_white_cross_right(void);
void hamster_white_cross_uturn(void);
void hamster_line_speed(double speed);
void hamster_line_tracer_speed(double speed);
void hamster_board_forward(void);
void hamster_board_left(void);
void hamster_board_right(void);
void hamster_leds(const char* left_color, const char* right_color);
void hamster_left_led(const char* color);
void hamster_right_led(const char* color);
void hamster_beep(void);
void hamster_buzzer(double hz);
void hamster_tempo(double bpm);
void hamster_pitch(const char* pitch);
void hamster_note(const char* pitch, double beats);
void hamster_io_mode_a(const char* mode);
void hamster_io_mode_b(const char* mode);
void hamster_output_a(double value);
void hamster_output_b(double value);
void hamster_open_gripper(void);
void hamster_close_gripper(void);
void hamster_release_gripper(void);
int hamster_signal_strength(void);
int hamster_left_proximity(void);
int hamster_right_proximity(void);
int hamster_left_floor(void);
int hamster_right_floor(void);
int hamster_acceleration_x(void);
int hamster_acceleration_y(void);
int hamster_acceleration_z(void);
int hamster_light(void);
int hamster_temperature(void);
int hamster_input_a(void);
int hamster_input_b(void);
int hamster_tilt(void);
int hamster_battery_state(void);

#define HAMSTER_S_ID "kr.robomation.physical.hamster.s"

#define HAMSTER_S_LEFT_WHEEL 0x00400000
#define HAMSTER_S_RIGHT_WHEEL 0x00400001
#define HAMSTER_S_BUZZER 0x00400002
#define HAMSTER_S_OUTPUT_A 0x00400003
#define HAMSTER_S_OUTPUT_B 0x00400004
#define HAMSTER_S_NOTE 0x00400008
#define HAMSTER_S_LINE_TRACER_MODE 0x00400009
#define HAMSTER_S_LINE_TRACER_SPEED 0x0040000a
#define HAMSTER_S_IO_MODE_A 0x0040000b
#define HAMSTER_S_IO_MODE_B 0x0040000c
#define HAMSTER_S_CONFIG_PROXIMITY 0x0040000d
#define HAMSTER_S_CONFIG_GRAVITY 0x0040000e
#define HAMSTER_S_CONFIG_BAND_WIDTH 0x0040000f

#define HAMSTER_S_LEFT_RGB 0x00E00100
#define HAMSTER_S_RIGHT_RGB 0x00E00101
#define HAMSTER_S_PULSE 0x00E00102
#define HAMSTER_S_SOUND 0x00E00103
#define HAMSTER_S_LINE_TRACER_GAIN 0x00E00104
#define HAMSTER_S_WRITE_SERIAL 0x00E00105
#define HAMSTER_S_MOTOR_MODE 0x00E00107

#define HAMSTER_S_SIGNAL_STRENGTH 0x00400010
#define HAMSTER_S_LEFT_PROXIMITY 0x00400011
#define HAMSTER_S_RIGHT_PROXIMITY 0x00400012
#define HAMSTER_S_LEFT_FLOOR 0x00400013
#define HAMSTER_S_RIGHT_FLOOR 0x00400014
#define HAMSTER_S_ACCELERATION 0x00400015
#define HAMSTER_S_LIGHT 0x00400016
#define HAMSTER_S_TEMPERATURE 0x00400017
#define HAMSTER_S_INPUT_A 0x00400018
#define HAMSTER_S_INPUT_B 0x00400019
#define HAMSTER_S_LINE_TRACER_STATE 0x0040001a
#define HAMSTER_S_TILT 0x0040001b
#define HAMSTER_S_BATTERY_STATE 0x0040001c

#define HAMSTER_S_FREE_FALL 0x00E00200
#define HAMSTER_S_TAP 0x00E00201
#define HAMSTER_S_READ_SERIAL 0x00E00202
#define HAMSTER_S_PULSE_COUNT 0x00E00203
#define HAMSTER_S_WHEEL_STATE 0x00E00204
#define HAMSTER_S_SOUND_STATE 0x00E00205
#define HAMSTER_S_SERIAL_STATE 0x00E00206

#define HAMSTER_S_LED_OFF "OFF"
#define HAMSTER_S_LED_RED "RED"
#define HAMSTER_S_LED_ORANGE "ORANGE"
#define HAMSTER_S_LED_YELLOW "YELLOW"
#define HAMSTER_S_LED_GREEN "GREEN"
#define HAMSTER_S_LED_SKY_BLUE "SKY BLUE"
#define HAMSTER_S_LED_BLUE "BLUE"
#define HAMSTER_S_LED_VIOLET "VIOLET"
#define HAMSTER_S_LED_PURPLE "PURPLE"
#define HAMSTER_S_LED_WHITE "WHITE"

#define HAMSTER_S_IO_MODE_ANALOG_INPUT "ANALOG INPUT"
#define HAMSTER_S_IO_MODE_DIGITAL_INPUT "DIGITAL INPUT"
#define HAMSTER_S_IO_MODE_DIGITAL_INPUT_PULL_UP "DIGITAL INPUT PULL UP"
#define HAMSTER_S_IO_MODE_DIGITAL_INPUT_PULL_DOWN "DIGITAL INPUT PULL DOWN"
#define HAMSTER_S_IO_MODE_VOLTAGE_INPUT "VOLTAGE INPUT"
#define HAMSTER_S_IO_MODE_SERVO_OUTPUT "SERVO OUTPUT"
#define HAMSTER_S_IO_MODE_PWM_OUTPUT "PWM OUTPUT"
#define HAMSTER_S_IO_MODE_DIGITAL_OUTPUT "DIGITAL OUTPUT"

#define HAMSTER_S_NOTE_C "C"
#define HAMSTER_S_NOTE_C_SHARP "C#"
#define HAMSTER_S_NOTE_D_FLAT "Db"
#define HAMSTER_S_NOTE_D "D"
#define HAMSTER_S_NOTE_D_SHARP "D#"
#define HAMSTER_S_NOTE_E_FLAT "Eb"
#define HAMSTER_S_NOTE_E "E"
#define HAMSTER_S_NOTE_F "F"
#define HAMSTER_S_NOTE_F_SHARP "F#"
#define HAMSTER_S_NOTE_G_FLAT "Gb"
#define HAMSTER_S_NOTE_G "G"
#define HAMSTER_S_NOTE_G_SHARP "G#"
#define HAMSTER_S_NOTE_A_FLAT "Ab"
#define HAMSTER_S_NOTE_A "A"
#define HAMSTER_S_NOTE_A_SHARP "A#"
#define HAMSTER_S_NOTE_B_FLAT "Bb"
#define HAMSTER_S_NOTE_B "B"

#define HAMSTER_S_SOUND_OFF "OFF"
#define HAMSTER_S_SOUND_BEEP "BEEP"
#define HAMSTER_S_SOUND_RANDOM_BEEP "RANDOM BEEP"
#define HAMSTER_S_SOUND_NOISE "NOISE"
#define HAMSTER_S_SOUND_SIREN "SIREN"
#define HAMSTER_S_SOUND_ENGINE "ENGINE"
#define HAMSTER_S_SOUND_CHOP "CHOP"
#define HAMSTER_S_SOUND_ROBOT "ROBOT"
#define HAMSTER_S_SOUND_DIBIDIBIDIP "DIBIDIBIDIP"
#define HAMSTER_S_SOUND_GOOD_JOB "GOOD JOB"
#define HAMSTER_S_SOUND_HAPPY "HAPPY"
#define HAMSTER_S_SOUND_ANGRY "ANGRY"
#define HAMSTER_S_SOUND_SAD "SAD"
#define HAMSTER_S_SOUND_SLEEP "SLEEP"
#define HAMSTER_S_SOUND_MARCH "MARCH"
#define HAMSTER_S_SOUND_BIRTHDAY "BIRTHDAY"

#define HAMSTER_S_TILT_FORWARD 1
#define HAMSTER_S_TILT_BACKWARD -1
#define HAMSTER_S_TILT_LEFT 2
#define HAMSTER_S_TILT_RIGHT -2
#define HAMSTER_S_TILT_FLIP 3
#define HAMSTER_S_TILT_NOT -3

#define HAMSTER_S_BATTERY_NORMAL 2
#define HAMSTER_S_BATTERY_LOW 1
#define HAMSTER_S_BATTERY_EMPTY 0

#define HAMSTER_S_MOTOR_AUTO "AUTO"
#define HAMSTER_S_MOTOR_WAVE "WAVE"
#define HAMSTER_S_MOTOR_HALF "HALF"
#define HAMSTER_S_MOTOR_FULL "FULL"

#define HAMSTER_S_DELIMITER_COMMA ","
#define HAMSTER_S_DELIMITER_COLON ":"
#define HAMSTER_S_DELIMITER_DOLLAR "$"
#define HAMSTER_S_DELIMITER_SHARP "#"
#define HAMSTER_S_DELIMITER_LINE "\r"

typedef struct hamster_s {
	const char* (*get_name)(void);
	void (*set_name)(const char* name);
	const char* (*get_id)(void);
	int (*get_index)(void);
	int (*e)(int device_id);
	int (*read)(int device_id);
	int (*read_at)(int device_id, int index);
	int (*read_array)(int device_id, int* data, int length);
	float (*read_float)(int device_id);
	float (*read_float_at)(int device_id, int index);
	int (*read_float_array)(int device_id, float* data, int length);
	int (*write_empty)(int device_id);
	int (*write)(int device_id, int data);
	int (*write_at)(int device_id, int index, int data);
	int (*write_array)(int device_id, const int* data, int length);
	int (*write_float)(int device_id, float data);
	int (*write_float_at)(int device_id, int index, float data);
	int (*write_float_array)(int device_id, const float* data, int length);
	void (*reset)(void);
	void (*dispose)(void);
	void (*motor_mode)(const char* mode);
	void (*wheels)(double left_velocity, double right_velocity);
	void (*left_wheel)(double velocity);
	void (*right_wheel)(double velocity);
	void (*stop)(void);
	void (*move_forward)(double cm, double velocity);
	void (*move_backward)(double cm, double velocity);
	void (*turn_left)(double degree, double velocity);
	void (*turn_right)(double degree, double velocity);
	void (*pivot_left_wheel)(double degree, double velocity);
	void (*pivot_right_wheel)(double degree, double velocity);
	void (*pivot_left_pen)(double degree, double velocity);
	void (*pivot_right_pen)(double degree, double velocity);
	void (*circle_left)(double degree, double radius, double velocity);
	void (*circle_right)(double degree, double radius, double velocity);
	void (*left_pen_circle_left)(double degree, double radius, double velocity);
	void (*left_pen_circle_right)(double degree, double radius, double velocity);
	void (*right_pen_circle_left)(double degree, double radius, double velocity);
	void (*right_pen_circle_right)(double degree, double radius, double velocity);
	void (*move_forward_sec)(double sec, double velocity);
	void (*move_backward_sec)(double sec, double velocity);
	void (*turn_left_sec)(double sec, double velocity);
	void (*turn_right_sec)(double sec, double velocity);
	void (*pivot_left_wheel_sec)(double sec, double velocity);
	void (*pivot_right_wheel_sec)(double sec, double velocity);
	void (*pivot_left_pen_sec)(double sec, double velocity);
	void (*pivot_right_pen_sec)(double sec, double velocity);
	void (*circle_left_sec)(double sec, double radius, double velocity);
	void (*circle_right_sec)(double sec, double radius, double velocity);
	void (*left_pen_circle_left_sec)(double sec, double radius, double velocity);
	void (*left_pen_circle_right_sec)(double sec, double radius, double velocity);
	void (*right_pen_circle_left_sec)(double sec, double radius, double velocity);
	void (*right_pen_circle_right_sec)(double sec, double radius, double velocity);
	void (*move_forward_pulse)(double pulse, double velocity);
	void (*move_backward_pulse)(double pulse, double velocity);
	void (*turn_left_pulse)(double pulse, double velocity);
	void (*turn_right_pulse)(double pulse, double velocity);
	void (*pivot_left_wheel_pulse)(double pulse, double velocity);
	void (*pivot_right_wheel_pulse)(double pulse, double velocity);
	void (*pivot_left_pen_pulse)(double pulse, double velocity);
	void (*pivot_right_pen_pulse)(double pulse, double velocity);
	void (*circle_left_pulse)(double pulse, double radius, double velocity);
	void (*circle_right_pulse)(double pulse, double radius, double velocity);
	void (*left_pen_circle_left_pulse)(double pulse, double radius, double velocity);
	void (*left_pen_circle_right_pulse)(double pulse, double radius, double velocity);
	void (*right_pen_circle_left_pulse)(double pulse, double radius, double velocity);
	void (*right_pen_circle_right_pulse)(double pulse, double radius, double velocity);
	void (*line_tracer_mode)(int mode);
	void (*line_left)(void);
	void (*line_right)(void);
	void (*line_both)(void);
	void (*cross_forward)(void);
	void (*cross_left)(void);
	void (*cross_right)(void);
	void (*cross_uturn)(void);
	void (*white_line_left)(void);
	void (*white_line_right)(void);
	void (*white_line_both)(void);
	void (*white_cross_forward)(void);
	void (*white_cross_left)(void);
	void (*white_cross_right)(void);
	void (*white_cross_uturn)(void);
	void (*line_gain_auto)(void);
	void (*line_gain)(double gain);
	void (*line_speed)(double speed);
	void (*board_forward)(void);
	void (*board_left)(void);
	void (*board_right)(void);
	void (*rgbs)(double left_red, double left_green, double left_blue, double right_red, double right_green, double right_blue);
	void (*left_rgb)(double red, double green, double blue);
	void (*right_rgb)(double red, double green, double blue);
	void (*leds)(const char* left_color, const char* right_color);
	void (*left_led)(const char* color);
	void (*right_led)(const char* color);
	void (*buzzer)(double hz);
	void (*tempo)(double bpm);
	void (*pitch)(const char* pitch);
	void (*note)(const char* pitch, double beats);
	void (*sound)(const char* sound);
	void (*sound_until_done)(const char* sound);
	void (*repeat_sound)(const char* sound, double repeat);
	void (*repeat_sound_until_done)(const char* sound, double repeat);
	void (*io_mode_a)(const char* mode);
	void (*io_mode_b)(const char* mode);
	void (*output_a)(double value);
	void (*output_b)(double value);
	void (*open_gripper)(void);
	void (*close_gripper)(void);
	void (*release_gripper)(void);
	void (*write_serial)(const char* text, int len);
	void (*write_serial_line)(const char* text, int len);
	void (*read_serial)(char* text, int len);
	void (*read_serial_ultil)(char* text, int len, char delimiter);
	void (*serial_rate)(double baud);
	int (*signal_strength)(void);
	int (*left_proximity)(void);
	int (*right_proximity)(void);
	int (*left_floor)(void);
	int (*right_floor)(void);
	int (*acceleration_x)(void);
	int (*acceleration_y)(void);
	int (*acceleration_z)(void);
	int (*light)(void);
	int (*temperature)(void);
	int (*input_a)(void);
	int (*input_b)(void);
	float (*voltage_a)(void);
	float (*voltage_b)(void);
	int (*free_fall)(void);
	int (*tap)(void);
	int (*pulse_count)(void);
	int (*tilt)(void);
	int (*battery_state)(void);
} HamsterS;

HamsterS* hamster_s_create(void);
HamsterS* hamster_s_create_port(const char* port_name);
const char* hamster_s_get_name(void);
void hamster_s_set_name(const char* name);
const char* hamster_s_get_id(void);
int hamster_s_e(int device_id);
int hamster_s_read(int device_id);
int hamster_s_read_at(int device_id, int index);
int hamster_s_read_array(int device_id, int* data, int length);
float hamster_s_read_float(int device_id);
float hamster_s_read_float_at(int device_id, int index);
int hamster_s_read_float_array(int device_id, float* data, int length);
int hamster_s_write_empty(int device_id);
int hamster_s_write(int device_id, int data);
int hamster_s_write_at(int device_id, int index, int data);
int hamster_s_write_array(int device_id, const int* data, int length);
int hamster_s_write_float(int device_id, float data);
int hamster_s_write_float_at(int device_id, int index, float data);
int hamster_s_write_float_array(int device_id, const float* data, int length);
void hamster_s_reset(void);
void hamster_s_dispose(void);
void hamster_s_motor_mode(const char* mode);
void hamster_s_wheels(double left_velocity, double right_velocity);
void hamster_s_left_wheel(double velocity);
void hamster_s_right_wheel(double velocity);
void hamster_s_stop(void);
void hamster_s_move_forward(double cm, double velocity);
void hamster_s_move_backward(double cm, double velocity);
void hamster_s_turn_left(double degree, double velocity);
void hamster_s_turn_right(double degree, double velocity);
void hamster_s_pivot_left_wheel(double degree, double velocity);
void hamster_s_pivot_right_wheel(double degree, double velocity);
void hamster_s_pivot_left_pen(double degree, double velocity);
void hamster_s_pivot_right_pen(double degree, double velocity);
void hamster_s_circle_left(double degree, double radius, double velocity);
void hamster_s_circle_right(double degree, double radius, double velocity);
void hamster_s_left_pen_circle_left(double degree, double radius, double velocity);
void hamster_s_left_pen_circle_right(double degree, double radius, double velocity);
void hamster_s_right_pen_circle_left(double degree, double radius, double velocity);
void hamster_s_right_pen_circle_right(double degree, double radius, double velocity);
void hamster_s_move_forward_sec(double sec, double velocity);
void hamster_s_move_backward_sec(double sec, double velocity);
void hamster_s_turn_left_sec(double sec, double velocity);
void hamster_s_turn_right_sec(double sec, double velocity);
void hamster_s_pivot_left_wheel_sec(double sec, double velocity);
void hamster_s_pivot_right_wheel_sec(double sec, double velocity);
void hamster_s_pivot_left_pen_sec(double sec, double velocity);
void hamster_s_pivot_right_pen_sec(double sec, double velocity);
void hamster_s_circle_left_sec(double sec, double radius, double velocity);
void hamster_s_circle_right_sec(double sec, double radius, double velocity);
void hamster_s_left_pen_circle_left_sec(double sec, double radius, double velocity);
void hamster_s_left_pen_circle_right_sec(double sec, double radius, double velocity);
void hamster_s_right_pen_circle_left_sec(double sec, double radius, double velocity);
void hamster_s_right_pen_circle_right_sec(double sec, double radius, double velocity);
void hamster_s_move_forward_pulse(double pulse, double velocity);
void hamster_s_move_backward_pulse(double pulse, double velocity);
void hamster_s_turn_left_pulse(double pulse, double velocity);
void hamster_s_turn_right_pulse(double pulse, double velocity);
void hamster_s_pivot_left_wheel_pulse(double pulse, double velocity);
void hamster_s_pivot_right_wheel_pulse(double pulse, double velocity);
void hamster_s_pivot_left_pen_pulse(double pulse, double velocity);
void hamster_s_pivot_right_pen_pulse(double pulse, double velocity);
void hamster_s_circle_left_pulse(double pulse, double radius, double velocity);
void hamster_s_circle_right_pulse(double pulse, double radius, double velocity);
void hamster_s_left_pen_circle_left_pulse(double pulse, double radius, double velocity);
void hamster_s_left_pen_circle_right_pulse(double pulse, double radius, double velocity);
void hamster_s_right_pen_circle_left_pulse(double pulse, double radius, double velocity);
void hamster_s_right_pen_circle_right_pulse(double pulse, double radius, double velocity);
void hamster_s_line_tracer_mode(int mode);
void hamster_s_line_left(void);
void hamster_s_line_right(void);
void hamster_s_line_both(void);
void hamster_s_cross_forward(void);
void hamster_s_cross_left(void);
void hamster_s_cross_right(void);
void hamster_s_cross_uturn(void);
void hamster_s_white_line_left(void);
void hamster_s_white_line_right(void);
void hamster_s_white_line_both(void);
void hamster_s_white_cross_forward(void);
void hamster_s_white_cross_left(void);
void hamster_s_white_cross_right(void);
void hamster_s_white_cross_uturn(void);
void hamster_s_line_gain_auto(void);
void hamster_s_line_gain(double gain);
void hamster_s_line_speed(double speed);
void hamster_s_board_forward(void);
void hamster_s_board_left(void);
void hamster_s_board_right(void);
void hamster_s_rgbs(double left_red, double left_green, double left_blue, double right_red, double right_green, double right_blue);
void hamster_s_left_rgb(double red, double green, double blue);
void hamster_s_right_rgb(double red, double green, double blue);
void hamster_s_leds(const char* left_color, const char* right_color);
void hamster_s_left_led(const char* color);
void hamster_s_right_led(const char* color);
void hamster_s_buzzer(double hz);
void hamster_s_tempo(double bpm);
void hamster_s_pitch(const char* pitch);
void hamster_s_note(const char* pitch, double beats);
void hamster_s_sound(const char* sound);
void hamster_s_sound_until_done(const char* sound);
void hamster_s_repeat_sound(const char* sound, double repeat);
void hamster_s_repeat_sound_until_done(const char* sound, double repeat);
void hamster_s_io_mode_a(const char* mode);
void hamster_s_io_mode_b(const char* mode);
void hamster_s_output_a(double value);
void hamster_s_output_b(double value);
void hamster_s_open_gripper(void);
void hamster_s_close_gripper(void);
void hamster_s_release_gripper(void);
void hamster_s_write_serial(const char* text, int len);
void hamster_s_write_serial_line(const char* text, int len);
void hamster_s_read_serial(char* text, int len);
void hamster_s_read_serial_ultil(char* text, int len, char delimiter);
void hamster_s_serial_rate(double baud);
int hamster_s_signal_strength(void);
int hamster_s_left_proximity(void);
int hamster_s_right_proximity(void);
int hamster_s_left_floor(void);
int hamster_s_right_floor(void);
int hamster_s_acceleration_x(void);
int hamster_s_acceleration_y(void);
int hamster_s_acceleration_z(void);
int hamster_s_light(void);
int hamster_s_temperature(void);
int hamster_s_input_a(void);
int hamster_s_input_b(void);
float hamster_s_voltage_a(void);
float hamster_s_voltage_b(void);
int hamster_s_free_fall(void);
int hamster_s_tap(void);
int hamster_s_pulse_count(void);
int hamster_s_tilt(void);
int hamster_s_battery_state(void);

#define TURTLE_ID "kr.robomation.physical.turtle"

#define TURTLE_LEFT_WHEEL 0x00900000
#define TURTLE_RIGHT_WHEEL 0x00900001
#define TURTLE_LED 0x00900002
#define TURTLE_BUZZER 0x00900003
#define TURTLE_PULSE 0x00900004
#define TURTLE_NOTE 0x00900005
#define TURTLE_SOUND 0x00900006
#define TURTLE_LINE_TRACER_MODE 0x00900007
#define TURTLE_LINE_TRACER_GAIN 0x00900008
#define TURTLE_LINE_TRACER_SPEED 0x00900009
#define TURTLE_LAMP 0x0090000a
#define TURTLE_LOCK 0x0090000b

#define TURTLE_SIGNAL_STRENGTH 0x0090000d
#define TURTLE_COLOR 0x0090000e
#define TURTLE_FLOOR 0x0090000f
#define TURTLE_ACCELERATION 0x00900010
#define TURTLE_TEMPERATURE 0x00900011
#define TURTLE_BUTTON 0x00900012
#define TURTLE_CLICKED 0x00900013
#define TURTLE_DOUBLE_CLICKED 0x00900014
#define TURTLE_LONG_PRESSED 0x00900015
#define TURTLE_COLOR_NUMBER 0x00900016
#define TURTLE_COLOR_PATTERN 0x00900017
#define TURTLE_PULSE_COUNT 0x00900018
#define TURTLE_WHEEL_STATE 0x00900019
#define TURTLE_SOUND_STATE 0x0090001a
#define TURTLE_LINE_TRACER_STATE 0x0090001b
#define TURTLE_TILT 0x0090001c
#define TURTLE_BATTERY_STATE 0x0090001d

#define TURTLE_COLOR_NONE -1
#define TURTLE_COLOR_BLACK 0
#define TURTLE_COLOR_RED 1
#define TURTLE_COLOR_ORANGE 2
#define TURTLE_COLOR_YELLOW 3
#define TURTLE_COLOR_GREEN 4
#define TURTLE_COLOR_SKY_BLUE 5
#define TURTLE_COLOR_CYAN 5
#define TURTLE_COLOR_BLUE 6
#define TURTLE_COLOR_PURPLE 7
#define TURTLE_COLOR_MAGENTA 7
#define TURTLE_COLOR_WHITE 8

#define TURTLE_LED_OFF "OFF"
#define TURTLE_LED_RED "RED"
#define TURTLE_LED_ORANGE "ORANGE"
#define TURTLE_LED_YELLOW "YELLOW"
#define TURTLE_LED_GREEN "GREEN"
#define TURTLE_LED_SKY_BLUE "SKY BLUE"
#define TURTLE_LED_BLUE "BLUE"
#define TURTLE_LED_VIOLET "VIOLET"
#define TURTLE_LED_PURPLE "PURPLE"
#define TURTLE_LED_WHITE "WHITE"

#define TURTLE_NOTE_C "C"
#define TURTLE_NOTE_C_SHARP "C#"
#define TURTLE_NOTE_D_FLAT "Db"
#define TURTLE_NOTE_D "D"
#define TURTLE_NOTE_D_SHARP "D#"
#define TURTLE_NOTE_E_FLAT "Eb"
#define TURTLE_NOTE_E "E"
#define TURTLE_NOTE_F "F"
#define TURTLE_NOTE_F_SHARP "F#"
#define TURTLE_NOTE_G_FLAT "Gb"
#define TURTLE_NOTE_G "G"
#define TURTLE_NOTE_G_SHARP "G#"
#define TURTLE_NOTE_A_FLAT "Ab"
#define TURTLE_NOTE_A "A"
#define TURTLE_NOTE_A_SHARP "A#"
#define TURTLE_NOTE_B_FLAT "Bb"
#define TURTLE_NOTE_B "B"

#define TURTLE_SOUND_OFF "OFF"
#define TURTLE_SOUND_BEEP "BEEP"
#define TURTLE_SOUND_RANDOM_BEEP "RANDOM BEEP"
#define TURTLE_SOUND_SIREN "SIREN"
#define TURTLE_SOUND_ENGINE "ENGINE"
#define TURTLE_SOUND_ROBOT "ROBOT"
#define TURTLE_SOUND_MARCH "MARCH"
#define TURTLE_SOUND_BIRTHDAY "BIRTHDAY"
#define TURTLE_SOUND_DIBIDIBIDIP "DIBIDIBIDIP"
#define TURTLE_SOUND_GOOD_JOB "GOOD JOB"

#define TURTLE_TILT_FORWARD 1
#define TURTLE_TILT_BACKWARD -1
#define TURTLE_TILT_LEFT 2
#define TURTLE_TILT_RIGHT -2
#define TURTLE_TILT_FLIP 3
#define TURTLE_TILT_NOT -3

#define TURTLE_BATTERY_NORMAL 2
#define TURTLE_BATTERY_LOW 1
#define TURTLE_BATTERY_EMPTY 0

typedef struct turtle {
	const char* (*get_name)(void);
	void (*set_name)(const char* name);
	const char* (*get_id)(void);
	int (*get_index)(void);
	int (*e)(int device_id);
	int (*read)(int device_id);
	int (*read_at)(int device_id, int index);
	int (*read_array)(int device_id, int* data, int length);
	float (*read_float)(int device_id);
	float (*read_float_at)(int device_id, int index);
	int (*read_float_array)(int device_id, float* data, int length);
	int (*write_empty)(int device_id);
	int (*write)(int device_id, int data);
	int (*write_at)(int device_id, int index, int data);
	int (*write_array)(int device_id, const int* data, int length);
	int (*write_float)(int device_id, float data);
	int (*write_float_at)(int device_id, int index, float data);
	int (*write_float_array)(int device_id, const float* data, int length);
	void (*reset)(void);
	void (*dispose)(void);
	void (*wheels)(double left_velocity, double right_velocity);
	void (*left_wheel)(double velocity);
	void (*right_wheel)(double velocity);
	void (*stop)(void);
	void (*move_forward)(double cm, double velocity);
	void (*move_backward)(double cm, double velocity);
	void (*turn_left)(double degree, double velocity);
	void (*turn_right)(double degree, double velocity);
	void (*pivot_left)(double degree, double velocity);
	void (*pivot_right)(double degree, double velocity);
	void (*circle_left)(double degree, double radius, double velocity);
	void (*circle_right)(double degree, double radius, double velocity);
	void (*move_forward_sec)(double sec, double velocity);
	void (*move_backward_sec)(double sec, double velocity);
	void (*turn_left_sec)(double sec, double velocity);
	void (*turn_right_sec)(double sec, double velocity);
	void (*pivot_left_sec)(double sec, double velocity);
	void (*pivot_right_sec)(double sec, double velocity);
	void (*circle_left_sec)(double sec, double radius, double velocity);
	void (*circle_right_sec)(double sec, double radius, double velocity);
	void (*move_forward_pulse)(double pulse, double velocity);
	void (*move_backward_pulse)(double pulse, double velocity);
	void (*turn_left_pulse)(double pulse, double velocity);
	void (*turn_right_pulse)(double pulse, double velocity);
	void (*pivot_left_pulse)(double pulse, double velocity);
	void (*pivot_right_pulse)(double pulse, double velocity);
	void (*circle_left_pulse)(double pulse, double radius, double velocity);
	void (*circle_right_pulse)(double pulse, double radius, double velocity);
	void (*line_tracer_mode)(int mode);
	void (*black_line)(void);
	void (*red_line)(void);
	void (*green_line)(void);
	void (*blue_line)(void);
	void (*any_line)(void);
	void (*black_line_until_red)(void);
	void (*black_line_until_yellow)(void);
	void (*black_line_until_green)(void);
	void (*black_line_until_sky_blue)(void);
	void (*black_line_until_blue)(void);
	void (*black_line_until_purple)(void);
	void (*black_line_until_any)(void);
	void (*red_line_until_black)(void);
	void (*green_line_until_black)(void);
	void (*blue_line_until_black)(void);
	void (*any_line_until_black)(void);
	void (*cross_forward)(void);
	void (*cross_left)(void);
	void (*cross_right)(void);
	void (*cross_uturn)(void);
	void (*line_gain)(double gain);
	void (*line_speed)(double speed);
	void (*rgb)(double red, double green, double blue);
	void (*led)(const char* color);
	void (*buzzer)(double hz);
	void (*tempo)(double bpm);
	void (*pitch)(const char* pitch);
	void (*note)(const char* pitch, double beats);
	void (*sound)(const char* sound);
	void (*sound_until_done)(const char* sound);
	void (*repeat_sound)(const char* sound, double repeat);
	void (*repeat_sound_until_done)(const char* sound, double repeat);
	void (*lamp)(int on);
	void (*lock)(int on);
	int (*signal_strength)(void);
	int (*floor)(void);
	int (*acceleration_x)(void);
	int (*acceleration_y)(void);
	int (*acceleration_z)(void);
	int (*temperature)(void);
	int (*button)(void);
	int (*clicked)(void);
	int (*double_clicked)(void);
	int (*long_pressed)(void);
	int (*color_number)(void);
	int (*is_color)(const char* color);
	int (*color_pattern)(void);
	int (*is_color_pattern)(const char* color1, const char* color2);
	int (*pulse_count)(void);
	int (*tilt)(void);
	int (*battery_state)(void);
} Turtle;

Turtle* turtle_create(void);
Turtle* turtle_create_port(const char* port_name);
const char* turtle_get_name(void);
void turtle_set_name(const char* name);
const char* turtle_get_id(void);
int turtle_e(int device_id);
int turtle_read(int device_id);
int turtle_read_at(int device_id, int index);
int turtle_read_array(int device_id, int* data, int length);
float turtle_read_float(int device_id);
float turtle_read_float_at(int device_id, int index);
int turtle_read_float_array(int device_id, float* data, int length);
int turtle_write_empty(int device_id);
int turtle_write(int device_id, int data);
int turtle_write_at(int device_id, int index, int data);
int turtle_write_array(int device_id, const int* data, int length);
int turtle_write_float(int device_id, float data);
int turtle_write_float_at(int device_id, int index, float data);
int turtle_write_float_array(int device_id, const float* data, int length);
void turtle_reset(void);
void turtle_dispose(void);
void turtle_wheels(double left_velocity, double right_velocity);
void turtle_left_wheel(double velocity);
void turtle_right_wheel(double velocity);
void turtle_stop(void);
void turtle_move_forward(double cm, double velocity);
void turtle_move_backward(double cm, double velocity);
void turtle_turn_left(double degree, double velocity);
void turtle_turn_right(double degree, double velocity);
void turtle_pivot_left(double degree, double velocity);
void turtle_pivot_right(double degree, double velocity);
void turtle_circle_left(double degree, double radius, double velocity);
void turtle_circle_right(double degree, double radius, double velocity);
void turtle_move_forward_sec(double sec, double velocity);
void turtle_move_backward_sec(double sec, double velocity);
void turtle_turn_left_sec(double sec, double velocity);
void turtle_turn_right_sec(double sec, double velocity);
void turtle_pivot_left_sec(double sec, double velocity);
void turtle_pivot_right_sec(double sec, double velocity);
void turtle_circle_left_sec(double sec, double radius, double velocity);
void turtle_circle_right_sec(double sec, double radius, double velocity);
void turtle_move_forward_pulse(double pulse, double velocity);
void turtle_move_backward_pulse(double pulse, double velocity);
void turtle_turn_left_pulse(double pulse, double velocity);
void turtle_turn_right_pulse(double pulse, double velocity);
void turtle_pivot_left_pulse(double pulse, double velocity);
void turtle_pivot_right_pulse(double pulse, double velocity);
void turtle_circle_left_pulse(double pulse, double radius, double velocity);
void turtle_circle_right_pulse(double pulse, double radius, double velocity);
void turtle_line_tracer_mode(int mode);
void turtle_black_line(void);
void turtle_red_line(void);
void turtle_green_line(void);
void turtle_blue_line(void);
void turtle_any_line(void);
void turtle_black_line_until_red(void);
void turtle_black_line_until_yellow(void);
void turtle_black_line_until_green(void);
void turtle_black_line_until_sky_blue(void);
void turtle_black_line_until_blue(void);
void turtle_black_line_until_purple(void);
void turtle_black_line_until_any(void);
void turtle_red_line_until_black(void);
void turtle_green_line_until_black(void);
void turtle_blue_line_until_black(void);
void turtle_any_line_until_black(void);
void turtle_cross_forward(void);
void turtle_cross_left(void);
void turtle_cross_right(void);
void turtle_cross_uturn(void);
void turtle_line_gain(double gain);
void turtle_line_speed(double speed);
void turtle_rgb(double red, double green, double blue);
void turtle_led(const char* color);
void turtle_buzzer(double hz);
void turtle_tempo(double bpm);
void turtle_pitch(const char* pitch);
void turtle_note(const char* pitch, double beats);
void turtle_sound(const char* sound);
void turtle_sound_until_done(const char* sound);
void turtle_repeat_sound(const char* sound, double repeat);
void turtle_repeat_sound_until_done(const char* sound, double repeat);
void turtle_lamp(int on);
void turtle_lock(int on);
int turtle_signal_strength(void);
int turtle_floor(void);
int turtle_acceleration_x(void);
int turtle_acceleration_y(void);
int turtle_acceleration_z(void);
int turtle_temperature(void);
int turtle_button(void);
int turtle_clicked(void);
int turtle_double_clicked(void);
int turtle_long_pressed(void);
int turtle_color_number(void);
int turtle_is_color(const char* color);
int turtle_color_pattern(void);
int turtle_is_color_pattern(const char* color1, const char* color2);
int turtle_pulse_count(void);
int turtle_tilt(void);
int turtle_battery_state(void);

#endif
